/* polyathlon.cpp 
// program by Dale Heatherington  started April 23, 2009
eeprom_read_block((void*)&line.data, (const void*)0, 9); 
// Version 2.1   June 17, 2009  (Add bluetooth serial commands and status)
// Modified for ATMEGA644 CPU.  This chip has 64K Program storage and 4K RAM.
// Note: This will not compile for the old ATMEGA8535 anymore.

 // 11/8/2012: Added lost line timer to the maze code so it can tolerate line gaps up to 1.5 inches and
 // be immune to camera flashes.
 
 //12/23/2013:  Changed code to compile as C++ code so it can call Arduino C++ libraries.
 //				Stole and modified Arduino I2C (TWI) libraries.
 //				Added C++ classes to handle 9DOF sensor stick from spark fun.
 //				Modified "rotate" and "drive straight" to use i2c gyro on 9 dof sensor stick
 

Compiled with AVR-GCC on Apple Mac Pro using Crosspack-AVR
 
Note: CPU type defined in Makefile but it must be a ATMEGA644
      Brownout protection mode must be set in the low fuse byte to prevent EEPROM corruption.
	  This is done in the makefile in the "pgm" section for the avrdude programmer
	  
	  Initially the line sensor photocell array must be calibrated on a white surface
	  by setting the mode switch to 8 and turning on power for 10 seconds.  Values are stored in EEPROM.
	  

This is a work in progress is may be incomplete or infested with bugs.
It has been tweeked to work with the hardware and sensors available and will 
probably not function on any other hardware.  It is presented for educational
purposes.



;;----------------------------------------------------------------------
;;This program is free software; you can redistribute it and/or modify  
;;it under the terms of the GNU General Public License as published by      
;;the Free Software Foundation; either version 2 of the License, or         
;;(at your option) any later version.                                       
;;
;;This program is distributed in the hope that it will be useful,       
;;but WITHOUT ANY WARRANTY; without even the implied warranty of            
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
;;GNU General Public License for more details.                              
                                                                          
;;You should have received a copy of the GNU General Public License     
;;along with this program; if not, write to the Free Software               
;;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
;;------------------------------------------------------------------------

*/
 
/* NOTE TO SELF:  float or double multiply,divide and add will execute in around 250uS.  
 */


extern "C" {
#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <avr/pgmspace.h>

#include <avr/eeprom.h>
#include <stdlib.h>
#include <string.h>
#include <avr/wdt.h>
}


//#define true 1
//#define false 0
#define F_CPU 16000000L  //16 MHZ CPU clock
extern "C" {
#include <util/delay.h>
}

//#include "twi.c"
//#include "twi.h"
#include "Wire.h"
#include "Imu.h"
//#include "Wire.cpp"





/* I/O and bit operation macros that have were 
   removed from late versions of avr-libc */

#define  _BV(bit)   (1 << (bit))
#define  inb(sfr)   _SFR_BYTE(sfr)
#define  inw(sfr)   _SFR_WORD(sfr)
#define  outb(sfr, val)   (_SFR_BYTE(sfr) = (val))
#define  outw(sfr, val)   (_SFR_WORD(sfr) = (val))
#define  cbi(sfr, bit)   (_SFR_BYTE(sfr) &= ~_BV(bit))
#define  sbi(sfr, bit)   (_SFR_BYTE(sfr) |= _BV(bit))

#define  loop_until_bit_is_set(sfr, bit)   do { } while (bit_is_clear(sfr, bit))
#define  loop_until_bit_is_clear(sfr, bit)   do { } while (bit_is_set(sfr, bit))
#define  outp(val, sfr)   outb(sfr, val)
#define  inp(sfr)   inb(sfr)
#define  BV(bit)   _BV(bit)
/*---------------------------------------------------------------------------------*/

#define WDTO_250MS 4        /* Watchdog timer value = 250ms */

#define TRUE 1
#define FALSE 0

#define MODE_STOP 0
#define REV 0
#define FWD 1
#define CW 2
#define CCW 1

#define LEFT 2
#define RIGHT 1
#define INTERSECTION 3
#define BOTH 3
#define NONE 0
#define MEM 99

#define MAX_FWD_SPEED -127
#define MAX_REV_SPEED 127

#define MAXTIME 180

//time in 10ms intervals to allow motor and bot to coast to a stop.
#define TMSTOP 5

//Threshold for IR edge detectors used as line detectors
//White is above this number, black below.
#define ET 0xb0

#define RF_BEACON
//#define OPTICAL_BEACON


/* Wheel encoder constants */

//Counts per inch integer (actually 36.9)
#define CPI 37
//Encoder counts required to rotate 360 degrees
#define C360  976
//Counts per degree integer part
#define Cdeg  2
//Counts per degree fractional part (hundredths)
#define Cfrac  71




//Workaround for C++ compiler bug that emits warnings about PROGMEM needing initialized variables
#define PROG_MEM __attribute__((section(".progmem.mydata"))) 
/* Define string constants in FLASH program memory */
const char testquad[] PROG_MEM = "TestQuad";
const char linefoll[] PROG_MEM = "LineFoll";
const char beaconfinder[]  PROG_MEM = "Beacon";
const char deadrec[]  PROG_MEM = "DeadRec";
const char bulldoz[]  PROG_MEM = "BullDoz";
const char linemaze[] PROG_MEM = "LineMaze";
const char whitelevel[] PROG_MEM = "WhiteLvl";
const char start[] PROG_MEM = "Start";
const char facebeacon[] PROG_MEM = "FaceBeac";

double testfloat;

/*--- Motor control variables and defs ----- */

//Bit in PORTB that enables drive motor h-bridges
#define MOT_ENAB 4

int r_speed;		//Motor speed control variables
int l_speed;
int speed;
uint8_t m_current;	//Motor current stored here
volatile uint8_t timer_mstop;

int xDist, yDist;

#define SMSIZE 100
char saveMaze[SMSIZE];   //Temporary maze intersection storage.
int choice;

//Prescalers used in timer interrupt to keep track of time.
uint8_t prescale ;
uint8_t prescale100;
uint8_t prescale1sec;
uint8_t prescaleMinute;
uint8_t prescale1ms;

uint8_t timer1sec;
volatile unsigned int minutes;
volatile int run_seconds,timer_trip, timer_move;
volatile int timer_led_yellow, timer1ms, timer10ms, timer100us, timer_overcurrent, timer_pid, timer_delay;
volatile uint8_t rnd;
volatile unsigned int runtime;  //Total runtime in 1/100 secs

//Quadrature encoder data
volatile  long odo_left, odo_right;
volatile  long trip_odo_right, trip_odo_left;

volatile long line_history_odo_right, line_history_odo_left;
volatile int speed_left, speed_right, l_motor, r_motor;
volatile uint8_t dir_left, dir_right,timer_ramp;
volatile int l_speedo_counter, r_speedo_counter, l_speedometer, r_speedometer;
uint8_t accl;   //Vary this to control motor acceleration.  1=max accl,  higher numbers less.
long gyroI;    //Gyro integral
volatile unsigned int timer_lost_line;


//TX UART
#define TXBLEN 80
char tx_buf[TXBLEN];
volatile uint8_t tx_ptr,tx_ctr ;
volatile uint8_t txstate;
char home[10] ;  //Cursor to home on local LCD display

//RX UART
#define RXBLEN 40
char rx_buf[RXBLEN];		//Store bytes here as they come in from serial port
char rx_data[RXBLEN];		//Move a compete line here for processing
volatile uint8_t rx_ptr, rxstate, rxsize;

//DEBUG
uint8_t debug[6];  	//Debug byte array - put data to be displayed on LCD here.

uint8_t last_sec;

uint8_t trip_counter;
uint8_t stop;
uint8_t sel_sw;
uint8_t edge_left, edge_right;  //Table edge detector analog values 0..255
int avg_contrast[4];
int *gyroOffset;

uint8_t   pot, pot2,  motor_current;

//Define  bit for PORTC object sensor mux address bits and enable
#define MUXENABLE 3
#define MUXA0 4
#define MUXA1 5
#define MUXA2 6



struct Line {	
	int centroid;
	int contrast;
	uint8_t brightness;
	uint8_t present;
	uint8_t mean;
	uint8_t map;
	uint8_t width;
	uint8_t data[9];  
	uint8_t data_wref[9];  //White refs from eeprom saved here.  last byte is checksum
	uint8_t maxwhite;
	uint8_t maxblack;
	int lost_line_timer;  //starts up counting when line is lost at 1000hz
	
};

struct Line line;

struct Beacon {
	int centroid;	
	uint8_t brightness;
	uint8_t left;
	uint8_t right;
	uint8_t maxbright;
	uint8_t minbright;
	
	
};

struct Beacon beacon;


struct IR {
	int centroid;
	uint8_t brightness;
	uint8_t contrast;
	uint8_t mean;
	uint8_t map;
	uint8_t data[6];
	uint8_t biasHist[4];   //Last 4 ambiant light bias values
	uint8_t alarm;		   //1=object in view, 0 if not
	uint8_t cntr;			//integration counter for alarm above
	
};

struct IR activeIR;

//PID data structure
struct PID {
	int kp;		//Porportional gain
	int p;		//Porportional
	int kd;		//deriviative gain
	int d;		//deriviative
	int ki;		//integral gain
	long i;		//integral
	int t;		//sampling time in milliseconds
	int ilim;  //Integrator high/low limits
	int u;		//updated pid output value
};

struct PID pid_line;		//Line follower PID structure instance
	

char sbuf[24];  //LCD bug tx buffer;

int samples[16];		//Holds 16 beacon sensor samples for correlator




// Analog to digital converter input defs
#define AD_LINE_FOLLOW 7
#define AD_POT  6
#define AD_POT2 5
#define AD_GYRO 5
#define AD_EDGE_PULSE 4
#define UNUSED 5
#define AD_BEACON_RIGHT 3
#define AD_BEACON_LEFT 2
#define AD_IR_PULSE  1
#define AD_MOTOR_CURRENT 0


uint8_t irsensor[8];
//Define Active IR LEDs in sensor irsensor[] array
#define REV_RIGHT 2
#define REV_LEFT 3
#define FWD_LEFT 1
#define FWD_RIGHT 0


int getContrast(uint8_t , uint8_t*, uint8_t*  );
uint8_t getMap(uint8_t ,  uint8_t*, uint8_t );

char DemoMode;    //Set to non-zero to enable testing closed loop vs open loop control

void odometer_debug();

/* EEPROM addresses */
#define MAZE_R 60
#define MAZE_L 160


//-----------------------------------------------------------------
// Actual code begins here.
//-----------------------------------------------------------------

// Waste time.
// t is in 10 ms intervals
void delay(int t)
{	int x = t;
	cli();
	timer_delay = t;
	sei();
	do{
		cli();
		x = timer_delay;
		sei();
	}
	while(x > 0);
	
}

//Wait 1 ms

void wait1ms(int n)
{   int x = n;
	cli();
	timer1ms = n;
	sei();
	
	do{
		cli();
		x = timer1ms;
		sei();
	}while(x > 0);	
	
}


//Wait n times 100us
void waitx100us(int n)
{   int x = n;
	cli();
	timer100us = n;
	sei();
	do{
		cli();
		x = timer100us;
		sei();
	}
   while( x > 0);
   
}

	
//------------------------------------------------------------------

//Convert 4 bit value to ascii hex format 0..f

char nibble(uint8_t c)
{	
	c = c & 0x0f;
	if(c < 10) return c + 0x30;   	//0..9
	return c + 0x37;		//A..F
}
	
//Convert 8 bit value to ascii string hex format 00..ff in buffer *cp
//A NULL will be appended to the end of the 2 digit string.
//Supplied buffer must be at least 3 bytes long
void bin2hex(char *cp, uint8_t i)
{  uint8_t t;

	t = i >> 4;
	cp[0] = nibble(t);
	t = i & 0x0f;
	cp[1] = nibble(t);
	cp[2] = 0x00;
		
}
//-------------------------------------------------------------------------


int hard_limit_range(int x, int lim_hi, int lim_lo)
{
   if(x < lim_lo) x = lim_lo; 
   if(x > lim_hi) x = lim_hi; 
   return x;

}

//Long int version
long hard_limit_range_l(long x, long lim_hi, long lim_lo)
{
	if(x < lim_lo) x = lim_lo; 
	if(x > lim_hi) x = lim_hi; 
	return x;
	
}

int get_adc(uint8_t chan)
{
	ADMUX = chan + BV(ADLAR);		//Select channel & left justify result
    sbi(ADCSRA,ADSC);                //Start conversion     
    while( bit_is_clear(ADCSRA,ADIF)); //Wait for ADC output registers to be loaded
	uint8_t adcl = ADCL >> 6 ;			//Must read ADCL first
	return (ADCH << 2) | (adcl >> 6) ;
}
//---------------------------------------------------------------------------

/* Read 10 bit AD converter but return 
   only 8 bits discarding low 2 bits. */

uint8_t get_adc8(uint8_t chan)
{  
    
    //sbi(PORTC,7);
    ADMUX = chan + BV(ADLAR);		//Select channel & left justify result
    sbi(ADCSRA,ADSC);                //Start conversion     
    while( bit_is_clear(ADCSRA,ADIF)); //Wait for ADC output registers to be loaded
	//cbi(PORTC,7);
	return ADCH;					//Return 8 bit result
	
}
//---------------------------------------------------------------------------
//Compute simple checksum of array of unsigned bytes.
uint8_t checksum(uint8_t *p, int n)
{   int i;
    uint8_t cksum = 0;
	for(i=0;i<n;i++) cksum += p[i] ;
	return cksum;
}

//---------------------------------------------------------------------------


//This is used to get the value of a multibyte volatile variable that is changed by interrupts.
//Called with pointer to the volatile long variable. Returns long.
long getAtomic(volatile long *x)
{	long rv;
	
	cli();		//Interupts off
	rv = *x;	// get value
	sei();		//Interrupts on
	return rv;	//return value
}

int getAtomicInt(volatile int *x)
{	int rv;
	
	cli();		//Interupts off
	rv = *x;	// get value
	sei();		//Interrupts on
	return rv;	//return value
}


//---------------------------------------------------------------------------
//Read an active IR seonsor
//Inputs: adr = address of IR LED 0..7
//
uint8_t readActiveIR(uint8_t adr)
{	
	adr = adr & 7;	
	
	if(adr & 1) sbi(PORTC,MUXA0); else cbi(PORTC,MUXA0);  //Address 1 of 8 IR LEDs
	if(adr & 2) sbi(PORTC,MUXA1); else cbi(PORTC,MUXA1);
	if(adr & 4) sbi(PORTC,MUXA2); else cbi(PORTC,MUXA2);
	
	cli();
	cbi(PORTC,MUXENABLE);		//Enable selected IR LED and wait 30uS
	_delay_us(30);
	sei();
	
	// Now either read the object detector or edge detector AD converter channel...
	// ...depending on which LED we are firing.  
	// Addresses 6 and 7 are for the Edge Detectors.
	// Addresses 0..5 are object detector LEDs.
	if(adr < 6) ADMUX = AD_IR_PULSE; 
		else ADMUX = AD_EDGE_PULSE;
	
	sbi(ADMUX,ADLAR);						//Left justify	
	sbi(ADCSRA,ADSC);						//Start conversion now
	
	cli();
	_delay_us(20);							//Wait about 20uS for sample and hold
	sei();	
	sbi(PORTC,MUXENABLE);							//IR pulse off
	
	while( bit_is_clear(ADCSRA,ADIF));		//Wait for ADC output registers to be loaded
	return ADCH;
	
		

}

//---------------------------------------------------------------------------
/* Compute centroid for the 6 channel active IR object detector.
 
*/

int getIRcentroid()
{
	uint8_t i ;
	unsigned int low, sumA, sumB;
	int centroid;
	
	low = 255;
	for( i=0;i<6;i++){		//Find the least bright reading  and store value in "low"
		if(activeIR.data[i] < low) low = activeIR.data[i];
	}
	
	sumA = sumB = 0;
	for( i=0;i<6;i++){				//Compute sums
		sumB += (activeIR.data[i] - low) * (i + 1);
		sumA += activeIR.data[i] - low;
	}
	
	centroid = (sumB * 16) / (sumA/2) ; //Compute and scale centroid  32 * (sumB/sumA)
	centroid = centroid - 112;			//Make it zero centered.
	
	return centroid;
	
	
	
}


//----------------------------------------------------------------------------

/* Read 6 channels of active IR to detect objects in the path.
 
   6 IR LEDs are turned on in sequence and the reflected light from each
	is measured with the photodiodes and amplifier.
    A photodiode reading is taken first without any IR LEDs turned on to
   get a zero reference.  This bias is subtracted from all 6 readings.
 
  Also, this finds the channel with maximum brightness, the centroid, and contrast.
  
 */

void refreshIRdata()
{
	int i,x;
	uint8_t max = 0;
	int bias;
	
	for(i=0;i<3;i++) activeIR.biasHist[i+1] = activeIR.biasHist[i];  //Shift the past bias values
	activeIR.biasHist[0] =  get_adc8(AD_IR_PULSE);  //Get new IR photodiode level for ambiant light (bias)
	bias = 0;
	for(i=0;i<4;i++) bias+=activeIR.biasHist[i];
	bias = bias / 4;   //Smoothed bias value
	
	for(i=0;i<6;i++){
		x = readActiveIR(i) - bias;  //Read all 6 active IR sensors and subtract bias
		if(x < 0) x = 0;
		activeIR.data[i] = (uint8_t)x;
		
	}
	
	for(i=0;i<6;i++){
		if(activeIR.data[i] > max) max = activeIR.data[i]; //Find max brightness for object sensors
	}
	
	
	
	
	activeIR.brightness = max;
	activeIR.centroid = getIRcentroid();
	activeIR.contrast = getContrast(6,activeIR.data,&activeIR.mean);
	activeIR.map = getMap(6,activeIR.data,activeIR.mean);
	
	if((activeIR.brightness > 0x1e) && (activeIR.cntr < 3)) activeIR.cntr++;
	if((activeIR.brightness <= 0x1e) && (activeIR.cntr > 0)) activeIR.cntr--;
	if(activeIR.cntr == 3) activeIR.alarm = 1;
	if(activeIR.cntr == 0) activeIR.alarm = 0;
	
	
	
		
	
	
	
}

//----------------------------------------------------------------------------

//Get table edge detector values
//Note that due to Op-AMP saturation issues on very bright surfaces I had to put 100us delays between
//successive readings to allow the OP-AMP to recover fully.

void refreshEdgeSensors()
{

	int bias,x;
	
	
	//Be sure we wait 100us for photodiode recovery since last reading.	
	_delay_us(100);  // waste 100us
	
	bias = get_adc8(AD_EDGE_PULSE);  //Get IR photodiode level for ambiant light
	x = readActiveIR(6) - bias;			//Get left edge detector
	if(x < 0) x = 0;
	edge_left = (uint8_t)x;
	
	_delay_us(100);				// 100us between readings
	
	x = readActiveIR(7) - bias;			//Get right edge detector	
	if(x < 0) x = 0;
	edge_right = (uint8_t)x;
		
	
}

//---------------------------------------------------------------------------

//Read 8 line tracking  photocells into line.data[] array.

//White levels for each cell are corrected using values stored in the EEPROM.
//See "saveLineWhiteLevels()" for how they were stored there.
//Note: Lower values equal brighter light.



void readLinePhotocells()
{
	uint8_t adr;
	int x;
	
	
	
	for(adr=0;adr<8;adr++){
		if(adr & 1) sbi(PORTC,MUXA0); else cbi(PORTC,MUXA0);
		if(adr & 2) sbi(PORTC,MUXA1); else cbi(PORTC,MUXA1);
		if(adr & 4) sbi(PORTC,MUXA2); else cbi(PORTC,MUXA2);
		
		_delay_us(20);							//Wait about 20uS for analog data to settle
		
		x = (int)get_adc8(AD_LINE_FOLLOW) - (int)line.data_wref[adr]; //Get data and subtract white level bias
		if(x < 0) x = 0;									//Do not allow negative values
		
		
		line.data[adr] = (uint8_t)(x & 255);  //store corrected value
		
		
	}
	
}

//-------------------------------------------------------------------------
//Compute line photocell centroid.  Assumes 8 values of brightness are in line.data[] array.
// Note: Lower values equal brighter light.
//
int getLineCentroid()
{   uint8_t i ;
	unsigned int low, sumA, sumB;
	int centroid;
	
	low = 255;
	for( i=0;i<8;i++){		//Find the brightest photocell and store value in "low"
		if(line.data[i] < low) low = line.data[i];
	}
	
	sumA = sumB = 0;
	for( i=0;i<8;i++){				//Compute sums  (relative to the "low")
		sumB += (line.data[i] - low) * (i + 1);
		sumA += line.data[i] - low;
	}
	
	centroid = (sumB * 16) / (sumA >> 1) ; //Compute and scale centroid  32*(sumB/sumA)
	centroid = centroid - 144;				//Make it zero centered
	line.centroid  = centroid;
	
	return centroid;
}


//-----------------------------------------------------------------------------
		
//-----------------------------------------------------------------------------
//Compute and return contrast and mean.
int getContrast(uint8_t n, uint8_t *data, uint8_t *mean )
{
	uint8_t high,low,i;
	int sliceLevel;
	unsigned int contrast;
	high = 0;
	low = 255;
	
	for(i=0;i<n;i++){
		if(data[i] > high) high = data[i];
		if(data[i] < low) low = data[i];
	}
	
	if(line.maxwhite > low) line.maxwhite = low;	//Global brightest white ever seen since power on  (lower value).
	if(line.maxblack < high) line.maxblack = high;  //darkest black ever seen (higher value)

	sliceLevel = (line.maxblack - line.maxwhite ) * 3 / 8 ;  //Find 3/8 difference between high and low
	sliceLevel = line.maxwhite + sliceLevel;   //Shift to match offset
	
	contrast = high - low;	
		   
	*mean = sliceLevel;
	
	return contrast;
	
}
//---------------------------------------------------------------------------


uint8_t getMap(uint8_t n, uint8_t* data, uint8_t mean)    //Photocells reading over the mean value 
						//are black and the assgned bit in "map" is set.
{						//Left most photocell is bit 7, right most is bit 0.
	uint8_t map = 0;
	uint8_t i;
	                  
	for(i=0;i<n;i++){
		map = map >> 1;
		if(data[i] > mean) map = map | 0x80;  
	}
	
	
	return map;
	
}

//---------------------------------------------------------------------------------

/* Get line photocell data and process it in various ways
 */
	
void refreshLineData()
{   int i, b;
	uint8_t c, threshold;
	
	line.lost_line_timer = timer_lost_line;  //Update local lost line timer from interrupt driven one.
	readLinePhotocells();
	line.contrast = getContrast(8,line.data,&line.mean);	
	line.map = getMap(8,line.data,line.mean);
	
	line.centroid = getLineCentroid();
	
	if(line.present == TRUE) threshold = 0x30 ; else threshold = 0x35;  //provide some hystereses
	
	line.present = FALSE;
	for(i=0;i < 8;i++){
		if(line.data[i] >= threshold) {
			line.present = TRUE;   //Higher value means less light ie: black line.
			timer_lost_line = 0;
			line.lost_line_timer = 0;
		}
	} 
	
		
	if(line.present == FALSE) {
		line.map = 0;
		line.centroid = 0;
		}
		
	line.width = 0;
	c = 1;
	for(i=0;i<8;i++){
		if(line.map & c ) line.width++;		//count the 1's in line.map to get width
		c = c << 1;		 
		
	}
		
	
	b = 0;
	for(i=0;i<8;i++) b += line.data[i];  //Sum all the photocells
	b = b / 8;  //get average
	line.brightness = 255 - b; //Invert to make brightness a positive function, 0-255
	
	
}

//------------------------------------------------------------------------------

//Computer centroid for the 2 beacon photocells

int getBeaconCentroid()
{   
	long  sumA, sumB;
	int centroid;
	
	
	//Compute sums
	sumB = beacon.left + (beacon.right  * 2);    //Max = 255 + (255*2) = 755
	sumA = beacon.left + beacon.right;			//Max = 255+255= 510
	
	
	centroid = (sumB * 200)  / sumA ; //Compute and scale centroid. 
	centroid = centroid - 300;
		
	return centroid;
}

//------------------------------------------------------------------------------
/*
//Read the left and right beacon photocells
//Compute brightness and get centroid.

void getBeacon()
{
	
beacon.left = 255 - get_adc8(AD_BEACON_LEFT);
beacon.right = 255 - get_adc8(AD_BEACON_RIGHT);
beacon.brightness = (beacon.right + beacon.left)  / 2;

beacon.centroid = getBeaconCentroid();

	
}
*/

//----------------------------------------------------------------------
//This is the 10.5 GHZ RF version
void getBeacon()
{
	
	beacon.left = get_adc8(AD_BEACON_LEFT);
	beacon.right = get_adc8(AD_BEACON_RIGHT);	
	beacon.brightness = (beacon.right + beacon.left)  / 2;
	
	beacon.centroid = getBeaconCentroid();
	
	
}


//----------------------------------------------------------------------

/* Adjust motor speeds.  */

#define CURRENT_LIMIT 127

/* Current limiter:
   1300 ohm motor current load resistor gives .5 volts per amp.
   25 AD converter units per amp.
   Value of 44 is 1.75 amps (sum of both motors)
   Value of 63 is 2.5 amps (sum of both motors)
*/

/* Locked Antiphase PWM Motor Speed Control at 28.8 KHZ.
   Note that a 50% duty cycle PWM waveform equals ZERO speed.
   Duty cycles above or below 50% run the motors either forward or reverse.
   eg: 75% is 1/2 speed forward while 25% is 1/2 speed reverse.
   
   Speed is set independently for left and right motors with l_speed and r_speed variables.
   The Range is -126 to +125 with zero being stop.   
   100% duty cycle is not allowed because the H-Bridge has bootstrap capacitors which require pulses
   at all times to work correctly.
   
*/

/*
void set_motor_speed(int l_speed, int r_speed){
      uint8_t hbridge_enable;
      
      int spd ;

      uint8_t lm_pwm, rm_pwm;				
     
      if((l_motor != 0) || (r_motor != 0)) timer_mstop = 40;  //400ms breaking when speed goes to zero
	
	speed_left = l_speed;		//Set target speed in speed ramp up/down interrupt routine
	speed_right = r_speed;		//   r_motor and l_motor get set in the ramp int routine.
     
      // Right motor PWM control 
       
		spd = r_motor;
       //if(abs(spd) > 2) spd = r_speed + ((r_speed - r_speedometer) * 4) ;  //speedometer feedback loop
	
		if(abs(r_motor) > 2) spd = r_motor + ((r_motor - r_speedometer) * 4) ;  //speedometer feedback loop

       spd = hard_limit_range(spd,125,-126);  //Limit range from +125 to -126 so h-bridge bootstrap always has pulses
       rm_pwm = (uint8_t)spd + 128;
       OCR1BH = 0;                      //set right PWM
       OCR1BL = rm_pwm;


       // Left motor PWM control 
       
		spd = l_motor;
       //if(abs(spd) > 2) spd = l_speed + ((l_speed - l_speedometer) * 4);  //speedometer feedback loop
		if(abs(l_motor) > 2) spd = l_motor + ((l_motor - l_speedometer) * 4);  //speedometer feedback loop
		

       spd = hard_limit_range(spd,125,-126);
       lm_pwm = (uint8_t)spd + 128; 
       OCR1AH = 0;                      //set left PWM
       OCR1AL = lm_pwm;

       // H-Bridge enable control 

       hbridge_enable = 1;      //Assume h-bridge will be enabled
                                //Then look for reasons to disable it below...

         if (m_current > CURRENT_LIMIT){
              hbridge_enable = 0;  // Limit drive motor current to 1.75 AMPs (sum of both motors)
         }

         if ((l_motor == 0)
             && (r_motor == 0)
             && (timer_mstop == 0)) hbridge_enable = 0;   //Disable H-Bridge if bot is stopped
	
	if(hbridge_enable == 0) cbi(PORTB,MOT_ENAB); else sbi(PORTB,MOT_ENAB);       

 }
*/

//---------------------------------------------------------------------------------
void set_motor_speed(int l_speed, int r_speed){	
		
	speed_left = l_speed;		//Set target speed in speed ramp up/down interrupt routine
	speed_right = r_speed;		//   PWM hardware gets set in interrupt code.	
	
	
}



//-----------------------------------------------------------------------------

void sendData(char *cp)
{
	while(txstate != 0);	//Wait until UART not busy      
	if(strlen(cp) < TXBLEN) strcpy(tx_buf,cp); else return;  //Copy string to tx buffer
	txstate = 4;         	//Tell scheduler it's ready to transmit
	sbi(UCSR0B, TXCIE0); 	//Enable TX UART interrupts  	
	UDR0 = cp[0];			//  stuff 1st byte into UART TX register to get it started
	
}


//-----------------------------------------------------------------------------------


   /* Send debug data to LCD display*/
  
void writelcd(char *cp){
      
	while(txstate != 0);	//Wait until UART not busy      
	if(strlen(cp) < TXBLEN) strcpy(tx_buf,cp); else return;  //Copy string to tx buffer
	
	txstate = 4;         	//Tell scheduler it's ready to transmit
	sbi(UCSR0B, TXCIE0); 	//Enable TX UART interrupts     
	cbi(UCSR0A,TXC0);		//Set transmit complete bit
	UDR0 = 0x12;			//  cursor to home 
   
      
   }
   
//-------------------
void clearScreen()
{   char s[] = {0x0c, 0};
    
	sendData(s);		//Clear LCD screen
}


    
//------------------------------------------------------------------
/* VT100 Terminal Cursor control 
  ESC [ y ; x H 
 
 */

void cursorPos(int x, int y)
{
	char ctlbuf[16];
	char xbuf[6];
	char ybuf[6];
	
	sprintf(xbuf,"%d",x);
	sprintf(ybuf,"%d",y);
	ctlbuf[0] = 27;
	ctlbuf[1] = '[';
	ctlbuf[2] = 0;
	strcat(ctlbuf,ybuf);
	strcat(ctlbuf,";");
	strcat(ctlbuf,xbuf);
	strcat(ctlbuf,"H");
	sendData(ctlbuf);
}

//----------------------------------------------------------------------
void cursorHome()
{	
	cursorPos(1,1);	
	
}

void cursorHomeLCD()
{
	char *s = (char*)"\x12\x0\x0";
	sendData(s);
	
}

//-----------------------------------------------------------------------------------

void printlcd(char *cp)
{   int i;
	char buf[20], *cp1,*cp2;
	for(i=0;i<20;i++) buf[i] = 0;
	clearScreen();
	cursorHomeLCD();
	cp1 = cp;
	cp2 = (char*)buf;
	
	//Copy 1st line of 8 chars to buffer
	for(i=0;i<8;i++) {if(*cp1 != 0) {*cp2 = *cp1; cp1++ ; cp2++ ;} else break;}
	if(*cp1 != 0){	
		*cp2 = '\r';		//Insert newline
		cp2++;
		*cp2 = '\n';
		cp2++;
		//Now copy the 2nd line of 8 chars
	    for(i=0;i<8;i++) { if(*cp1 != 0){ *cp2 = *cp1; cp1++ ; cp2++;} else break;}
	}
	
	//Send buffer to LCD display
	sendData(buf);
	
	
}

	
//----------------------------------------------------------------------
   void refreshDebug()
   {	int i,j;
	   
   
   for(i=0,j=0;i<6;i++){
	   bin2hex(sbuf+j,debug[i]);  //Convert all 6 debug values to ascii hex in the sbuf array
	   sbuf[j+2] = ' ';
	   j = j+3;
           if(i == 2) {        //Next line on LCD display after 8 chars.
             sbuf[j++] = '\r';
             sbuf[j++] = '\n';
           }
   }
	   sbuf[j] = 0;
	   //cursorHome();			//For bluetooth serial debug VT100 emu
	   cursorHomeLCD();
   	   sendData(sbuf);
	   
	   

   }


//----------------------------------------------------------------------

char debugReady()
{
	if(txstate != 0) return 0;
	else return 1;
}
	
//----------------------------------------------------------------------

// Wait t*10ms and display time remaining on LCD display

void delay_start(int t, PGM_P s){
	int tenths;
	char home[2] = {0x12,0};
	
	timer10ms = t;	
		
	while(timer10ms > 0 ){		//Wait here for t*10ms  and show time remaining on LCD
	  
	  tenths = timer10ms / 10;   //Converts hundredths to tenths.
	  
	  if(txstate == 0){
		itoa(tenths,sbuf,10);		//Convert integer to decimal string
		if(tenths > 9){
				sbuf[2] = sbuf[1];
				sbuf[1] = '.';		//Insert decimal point
							
			}
			else
			{
				sbuf[2] = sbuf[0];
				sbuf[1] = '.';
				sbuf[0] = '0';		//Insert leading zero and decimal point
				
			}
		 
		  sbuf[3] = '\r';
		  sbuf[4] = '\n';
		  strcpy_P(&sbuf[5], s);  //Copy Title string from Program memory to ram buffer
		  
		  sendData(home);			//Cursor to home
		  
		  sendData(sbuf);			//Write to LCD display
		
		}
	}
	runtime = 0;   //Reset timer 1/100 sec
	  
  }
//----------------------------------------------------------------------
//Unhandled interrupts come here
ISR(BADISR_vect)
{
	sendData(home);			//Cursor to home
	
	sendData((char*)"Bad intr");			//Write to LCD display
	while(1 == 1);
	
   
}


//----------------------------------------------------------------------
/* UART receive interrupt handler */

SIGNAL(USART0_RX_vect)
{
	
	
    uint8_t ch,i;
    
    ch = UDR0;									//Get the current rx char
	if(rx_ptr < RXBLEN)  rx_buf[rx_ptr++] = ch;		//put it in buffer
    if(ch == 0x0d){
		if(rxstate == 0) {
			for(i=0;i<rx_ptr;i++) rx_data[i] = rx_buf[i];  //Copy uart buffer to rx data buffer
			rx_data[rx_ptr] = 0;							// Append a NULL to make this a C string
			rxsize = rx_ptr;
			rxstate = 1;									//Set rxstate to 1 (data available)
		}
		
		rx_ptr = 0;				//Reset buffer pointer to 0
	}
		
		
}


//----------------------------------------------------------------------


/* UART Transmit interrupt handler state machine
 
 To start a transmission of chars in tx_buf, 
 set txstate to 4, enable TX Complete Interrupt
 and write the 1st character to UDR0  
 
 */ 

SIGNAL(USART0_TX_vect )

{ 
   

      switch(txstate) {
      case 0:  	break;
      case 1:	//unused
      case 2:	//unused
      case 3: 	//unused
      case 4: 	tx_ptr = 1;
         	tx_ctr = strlen(tx_buf)-1; //Compensate for 1st char stuffed in before interrupt happened.
			  if(tx_ctr > 0) txstate = 5;
			  else{
				  txstate = 0;
				  break;
			  }
			  
		//fall through...

      case 5:  	UDR0 = tx_buf[tx_ptr++] ;  //Now send buffer contents
            	tx_ctr--;
            	if (tx_ctr == 0){ 
               	   txstate = 6;
               
            	}
         	
            break;

      case 6:  
				txstate = 0;
				cbi(UCSR0B,TXCIE0);		// Clear TX complete interrupt enable bit
				sbi(UCSR0A,TXC0);    //Set transmit complete bit
	
               break;



      }

      
}

//----------------------------------------------------------------------------

/* Come here 10,000 times per second.   
   Process various software timers.
   100us resolution.
 
 Note: Timer variables used here must be declared volatile.
 
 */
SIGNAL(TIMER2_COMPA_vect )
{
	int spd;
	uint8_t hbridge_enable, rm_pwm, lm_pwm; 
		
		
		rnd++ ;
		rnd = rnd ^ beacon.left;   //Make a random number
		
		
		if(timer100us > 0) timer100us--;		//General purpose 100us/tick timer
		if(timer_pid > 0) timer_pid--;			//100us per tick pid timer
		if(timer_ramp > 0) timer_ramp--;		//Speed acceleration ramp timer
	
	/*Motor Speed Control */
		if(l_speedo_counter < 1000) l_speedo_counter++; 	
		if(r_speedo_counter < 1000) r_speedo_counter++; 
		l_motor = speed_left;
		r_motor = speed_right;
		if((l_motor != 0) || (r_motor != 0)) timer_mstop = 40; /* 400ms timeout on timer_mstop*/
	
		hbridge_enable = 1;      //Assume h-bridge will be enabled
		//Then look for reasons to disable it below...
		
		if (m_current > CURRENT_LIMIT){
			hbridge_enable = 0;  // Limit drive motor current to CURRENT_LIMIT AMPs (sum of both motors)
		}
		
		if ((l_motor == 0)
			&& (r_motor == 0)
			&& (timer_mstop == 0)) hbridge_enable = 0;   //Disable H-Bridge if bot is stopped
		
		if(hbridge_enable == 0) cbi(PORTB,MOT_ENAB); else sbi(PORTB,MOT_ENAB);       
		
		spd = r_motor;		
		if((abs(r_motor) > 2))  spd = r_motor + ((r_motor - r_speedometer) * 4) ;  //speedometer feedback loop
	    
		spd = hard_limit_range(spd,125,-126);  //Limit range from +125 to -126 so h-bridge bootstrap always has pulses		
		rm_pwm = (uint8_t) spd + 128;
		OCR1BH = 0;                      //set right PWM
		OCR1BL = rm_pwm;
		
		spd = l_motor;		
		if(abs(l_motor) > 2) spd = l_motor + ((l_motor - l_speedometer) * 4) ;  //speedometer feedback loop		
		spd = hard_limit_range(spd,125,-126);  //Limit range from +125 to -126 so h-bridge bootstrap always has pulses		
		lm_pwm = (uint8_t) spd + 128; 
		OCR1AH = 0;                      //set left PWM
		OCR1AL = lm_pwm;
	 //End of motor control
	
	// 1ms timers
	if(--prescale1ms == 0){
		prescale1ms = 10;
		if(timer1ms > 0) timer1ms--;
		if(timer_lost_line < 0xffff) timer_lost_line++;  //Counts up when line is not seen
		gyro.tick();	//1ms Gyro timer
		accel.tick();   //1ms accelerometer timer
	}
	
	// 10ms timers

		if(--prescale == 0){      //This executes 100 times per sec.
			prescale = 100;
         
			if(prescale1sec > 0) prescale1sec--;
			if(timer10ms > 0) timer10ms--;
			if(timer_trip > 0) timer_trip--;
			if(timer_mstop > 0) timer_mstop--;
			if(timer_overcurrent > 0) timer_overcurrent--;
			
			if(timer_move > 0) timer_move--;
			if(timer_delay > 0) timer_delay--;
		    runtime++;							//runtime in 1/100 secs
			}

      
		if(prescale1sec == 0){  //This executes once per second
			prescale1sec = 100;
			
			if(prescaleMinute > 0) prescaleMinute--;
			run_seconds++;
			
        
			}

		if(prescaleMinute == 0){ //This executes once per minute
			prescaleMinute = 60;
			minutes++;        
			}

		 
    /* End interrupt code */
    

}

//-------------------------------------------------------------------------

/* Quadrature encoder interrupt handlers */

/*  36.8  counts per inch  or 0.0272 inch per count  */

//LEFT side quadrature clock
SIGNAL(INT0_vect)
{
	l_speedometer = 768 / l_speedo_counter;
	l_speedo_counter = 1;
	
	if(PIND & 0x80) dir_left = FWD; else dir_left = REV;
	if(dir_left == FWD){
		odo_left++; 
		trip_odo_left++;
	}else {
		odo_left--;
		trip_odo_left--;
		l_speedometer = -l_speedometer;
	}
	
	
	
}

//Right side Quadrature clock
SIGNAL(INT1_vect)
{
	r_speedometer = 768 / r_speedo_counter;
	r_speedo_counter = 1;					//reset and avoid a divide by zero situation
	
	if(PIND & 0x40) dir_right = FWD; else dir_right = REV;  
	if(dir_right == FWD){
		odo_right++; 
		trip_odo_right++;
	}else {
		odo_right--;
		trip_odo_right--;
		r_speedometer = -r_speedometer;
	}
	
	
	
	
}


//------------------------------------------------------------------------

//Read pot and motor current values.
//Also beacon values.

void refresh_sensors(void)
{
	pot2 = get_adc8(AD_POT2);
 	pot = get_adc8(AD_POT);	
	m_current = get_adc8(AD_MOTOR_CURRENT);
	
	getBeacon();
	
	
		
}	
	


//--------------------------------------------------------------------------

//Returns true if motors have been pulling excessive current for too long
//Unused in this applicaton
/*
uint8_t over_current(void)
{

	motor_current = get_adc8(AD_MOTOR_CURRENT);
	
	if (motor_current < CURRENT_LIMIT) timer_overcurrent = 60;  //Reset timer to 600ms if no overcurrent
	
	if ((timer_overcurrent == 0) && (motor_current >= CURRENT_LIMIT)) return 1;
	
	return 0;
	
	}

*/


//-------------------------------------------------------------------------
//Correlator
//Enter with most recent sensor value.
//Do 16 sample correlation with square wave and return correlation value.
//Sq wave has 8 units positive and 8 units negative.

int correlate(int s)
{   uint8_t i;
	int a;

	for(i=15;i > 0; i--) samples[i] = samples[i-1];  // shift old data by 1 place in 16 integer array
	samples[0] = s;									// put in new data
	a = 0;											// Clear accumulator
	for(i=0;i<8;i++) a = a - samples[i];			// sum the negative half
	for(i=0;i<8;i++) a = a + samples[i+8];			// sum the positive half
	
	return a / 16;									// return correlation value divided by 16
	
}
	



//---------------------------------
/*
#ifdef RF_BEACON

// If mem is 0 then we spin 360 degrees to find the beacon.  
// If mem is 1 we retrive stored beacon direction from eeprom.

int face_beacon(char test, char mem)
{  uint8_t i;
	int deltaLR,  maxc, lastd, speed;
	long odo_mark_beacon, odo_next_sample;
	maxc = 0;
	
	
	if(test == TRUE) delay_start(500,facebeacon);   //Wait 5 secs. and put beacon message on LCD
	
	
	timer10ms  = 200;
	refresh_sensors();
	speed = 70;    //pot / 2; //Use pot for testing
	set_motor_speed(-speed, speed);  //Spin CCW
		
	getBeacon();
	beacon.minbright = 255;
	beacon.maxbright = 0;
	deltaLR = beacon.left - beacon.right;
	lastd = deltaLR;
	timer100us = 14;
	
	for(i=0;i<16;i++) samples[i] = 0;  // Clear correlator
	
	cli();
	trip_odo_left = trip_odo_right = 0;
	sei();
	
	odo_mark_beacon = 0;
	odo_next_sample = 0; 
	
	if(mem == 1) { 
		cli();
		trip_odo_left = trip_odo_right = C360; 
		odo_mark_beacon = (long)eeprom_read_dword((const uint32_t *)32)	;  // retrive previous run beacon angle from eeprom	
		sei();
	}
	
	  
	if(mem == 0){					  
							
	
		while(getAtomic(&trip_odo_right) < C360){  // Rotate for 360 degrees 
		
			if(getAtomic(&trip_odo_right) >= odo_next_sample ){	//Take RF level readings every 4 degrees				
				odo_next_sample = getAtomic(&trip_odo_right) + 13 ;   // aprox. 5 degrees rotation per sample	
				
				getBeacon();		//get latest RF beacon signal level readings
			
				int x = beacon.brightness - abs(beacon.left - beacon.right);
				if(x > beacon.maxbright){
					beacon.maxbright = x;					
					odo_mark_beacon = getAtomic(&trip_odo_right) ;
					
				}
				
			
				
			
				if(beacon.brightness < beacon.minbright) beacon.minbright = beacon.brightness;  //Get min and max brightness levels
			
			
			}
			
		}
		
	}
	

	
	set_motor_speed(0,0);   //Stop motor to avoid sudden reversal
	delay(TMSTOP);				//Wait  for bot to stop (50ms)
	

	if(mem == 0) {
		eeprom_write_dword(	(uint32_t *) 32, (uint32_t) odo_mark_beacon);  // Save the beacon direction in eeprom
	}

	
	//calculate the shortest way to get back facing the beacon, CW or CCW.
	
	
	if((getAtomic(&trip_odo_right) - odo_mark_beacon)	< (C360/2) ){
	
		set_motor_speed(speed, -speed);	
		
		while((getAtomic(&trip_odo_right)-50) > odo_mark_beacon);  //rotate CW back to point at beacon. -50 compensates for overshoot
	}
	else{
		
		set_motor_speed(-speed,speed);	
		
		while((getAtomic(&trip_odo_right)+50) < (C360 + odo_mark_beacon));  //Rotate CCW to face beacon. +50 compensates for overshoot
	}
	
		
	
	set_motor_speed(0,0);
	delay(TMSTOP);				//Delay 50 ms for bot to stop
	
	if(test == TRUE){
		delay(15);
		
		debug[0] = maxc & 255;
		debug[1] = trip_odo_right >> 8;
		debug[2] = trip_odo_right & 255;
		debug[3] = beacon.maxbright;
		debug[4] = odo_mark_beacon >> 8;
		debug[5] = odo_mark_beacon & 255;
		refreshDebug();
		while(1 == 1);    //Stall here forever if in test mode.
	}
	
	if(mem == 1){
		getBeacon();		//get  RF beacon signal level readings at this position ( Hopeflly facing beacon)
		beacon.maxbright = beacon.brightness;
	}
	
	return beacon.maxbright;
	
}


#endif
*/

int face_beacon(char test, char mem)
{
    int zcount, i, u;
	
	
	refresh_sensors();
	//int p = pot / 32;  //debug
	zcount = 0;
			
	u = beacon.centroid;
	if(u > 0) u = 120; else u = -120;
	set_motor_speed(u,-u);
	
	delay(15);			//Wait 150ms to turn about 45 deg or so.	
	i = 750;			//750ms timeout for next operation
	
	while(--i > 0) {
		
		refresh_sensors();		
		u = beacon.centroid * 3;
		hard_limit_range(u,70,-70);
		set_motor_speed(u,-u);
		if(abs(beacon.left - beacon.right) <= 4  ) zcount++; else { if(zcount > 0) zcount--;}
			
		if(zcount > 15) break;
		wait1ms(1);						//Force loop to execute at 1ms intervals
		
	}
	
	
	
	/*
	
	set_motor_speed(0,0);
	debug[0] = beacon.brightness;
	debug[1] = abs(beacon.right-beacon.left);
	debug[2] = beacon.centroid ;
	debug[3] = zcount & 255;
	debug[4] = i >> 8;
	debug[5] = i & 255;
	refreshDebug();	
	while(1);
	 */
	
	return(beacon.brightness);
}


//------------------------------------------------------------------------------

// Update PID control value.  
// Input is latest error term "p" and pointer to PID data structure "*pid" .



int computePID(int p, struct PID *pid)
{
	  
	int il = 0;
	
		
	if(getAtomicInt(&timer_pid) == 0){
		timer_pid = pid->t * 10;	 // t is given in 1ms increments, timer is 0.1 ms per tick
		
		pid->d = (p - pid->p) * pid->kd ;		//compute differencial d * kd
		pid->d = pid->d / pid->t;			//Divide by delta t
		
		pid->p = p;
		
		il = hard_limit_range(p,50,-50);  //Limit integrator drive level to reduce windup
		il = il * pid->t;					//mpy by delta t
		pid->i = (il + pid->i);				//compute integral i 		
		il = pid->i / pid->ki;				//Divide by scaling factor ki
		il = hard_limit_range(il,pid->ilim,-pid->ilim);  //Limit max range of integral
		
		pid->u = (pid->p * pid->kp) + pid->d + il;
			
	}
	
	return pid->u;
}

//---------------------------------------------------------------------------------------

/* PID line tracking.   Adjusts l_speed and r_speed but does not call set_motor_speed() 
 Caller must use set_motor_speed(l_speed,r_speed);   */

void trackLine(int speed)
{	long spsq;
	int l_speed, r_speed;
	
	refreshLineData();
	
	if((line.width > 0) && (line.width < 3)){  //If we see a normal thin line, track it
		
		int u = computePID(line.centroid,&pid_line);  //Update PID feedback loop			
		
		spsq =  (speed * speed) / 128 ;			// Speed squared if above 64
		if(speed < 64) spsq = speed / 2;		//Or just porportional if speed is below 64
		u = (u * spsq) / 150 ;
		u = hard_limit_range(u,speed, -speed);
		r_speed = speed + u;					//Apply updated error term
		l_speed = speed - u;
		

		
	} else {l_speed = speed; r_speed = speed;}  //else fat line. Assume 90 deg corner or intersection. Don't track.
	
	
	set_motor_speed(l_speed, r_speed);
	
}


//-------------------------------------------------------------------------------

void searchForLine(int turn, int time, int speed)
{
	timer10ms = time;
	if(turn > 0){ set_motor_speed(speed,-speed);}
	else { set_motor_speed(-speed,speed);}
	
	do{
		refreshLineData();
	}while(( line.present == FALSE) && (timer10ms > 0));
	
	set_motor_speed(0,0);
	
}


//------------------------------------------------------------------------------

//PID feedback control is used to quickly center on the line.

void centerOnLine(int t, int spd)
{
	int   u;
		
	
	
	
	timer10ms = t;					//Allow  time in 10ms per tick to settle on line
	
	//Now use PID to center on the line.
	while(timer10ms > 0){
		refreshLineData();	
		//if(line.map & 0x18) break;				//Quit when we get centered
		u = computePID(line.centroid,&pid_line);		
		u = hard_limit_range(u,spd, -spd);
		set_motor_speed(-u,u);					//servo  to line center
		
	}
	
	set_motor_speed(0,0);						//Stop.
	
	
}


//------------------------------------------------------------------------------

void odometer_debug()
{
	if(debugReady()){
	debug[0] = (trip_odo_right >> 16) & 255;
	debug[1] = (trip_odo_right >> 8) & 255;
	debug[2] = (trip_odo_right & 255);
	debug[3] = (trip_odo_left >> 16) & 255;
	debug[4] = (trip_odo_left >> 8) & 255;
	debug[5] = (trip_odo_left & 255);
	refreshDebug();
    }
}
	
//------------------------------------------------------------------------------
void driveStraight(int distance, int speed, char direction)
{   long dcnt;
	long d;
	int spd;
	char a = FALSE;
	
	cli();
	trip_odo_left = trip_odo_right = 0;	
	dcnt  = CPI * distance;   //d now has odometer counts required
	sei();
	spd = 0;
	timer_ramp = 50;
	
	do{
		
		if(timer_ramp == 0){				//Ramp up speed slowly when starting
			if(spd < speed) spd++;
			timer_ramp = 100;				//100ms per step
		}
		
		
		cli();
		d = (dcnt - abs(trip_odo_left)) / 4;			//Ramp down speed when we get near end
		sei();
		d = hard_limit_range_l(d,spd,-spd);
		if((d < 20) && (d > 0)) d = 20;			//set minimum speed
		if((d > -20) && (d < 0)) d = -20;
		spd = (int)d;
		
		
		
		
		refresh_sensors();
		
		cli();
		d = abs(trip_odo_right) - abs(trip_odo_left);
		
		sei();
		if(d > 5) d = 5;    //Limit odometer difference range to +/- 5
		if(d < -5) d = -5;
		
				
		l_speed = spd + (d * 8);    //Correct steering based on odometer count difference.
		r_speed = spd - (d * 8);
		
		if(direction == 'R') {
			l_speed = -l_speed;
			r_speed = -r_speed;
		}
		 set_motor_speed(l_speed, r_speed);
		
		//odometer_debug();
		cli();
		if(abs(trip_odo_left) >= dcnt) a = TRUE;
		sei();
		
		 
	}while(a == FALSE);
	
	
	set_motor_speed(0, 0);
	delay(TMSTOP);				//wait 50ms for motor to stop
		
		
	
}//------------------------------------------------------------------------------
void driveStraightGyro(int distance, int speed, char direction)
{   long dcnt;
	long gyroI,d;
	int spd;
	char a = FALSE;
	
	
	gyroI = 0;		
		
	cli();
	trip_odo_left = trip_odo_right = 0;	
	dcnt  = CPI * distance;   //d now has odometer counts required
	sei();
	spd = 0;
	timer_ramp = 50;
	
	
	
	do{
		
		if(timer_ramp == 0){				//Ramp up speed slowly when starting
			if(spd < speed) spd++;
			timer_ramp = 100;				//100ms per step
		}
		
		
		cli();
		d = (dcnt - abs(trip_odo_left)) / 4;			//Ramp down speed when we get near end
		sei();
		d = hard_limit_range_l(d,spd,-spd);
		if((d < 20) && (d > 0)) d = 20;			//set minimum speed
		if((d > -20) && (d < 0)) d = -20;
		spd = (int)d;
		
				
		refresh_sensors();
			
		gyro.read();	
		gyroI += gyro.z();;
				
		d = gyroI / 3000;
		
		
		if(d > 5) d = 5;    //Limit  difference range to +/- 5
		if(d < -5) d = -5;
		
		
		l_speed = spd + d;    //Correct steering based on gyro integrator .
		r_speed = spd - d;
		
		if(direction == 'R'){
			l_speed = -spd + d;    //REVERSE only.  Correct steering based on gyro integrator .
			r_speed = -spd - d;
		}
		
		set_motor_speed(l_speed, r_speed);
		
		//odometer_debug();
		cli();
		if(abs(trip_odo_left) >= dcnt) a = TRUE;
		sei();
		
		
	}while(a == FALSE);
	
	
	set_motor_speed(0, 0);
	delay(50);				//wait 500ms for motor to stop
	
	
	
}



//----------------------------------------------------------------------------
//Wheels are 7.875 inches apart.  
//To turn 360 degrees requires they each travel the circumference which is pi*d = 24.74 inches.
//with 36.9 quadrature counts per inch they need to travel 36.9 * 24.74 = 912.9 counts
// or 2.536 counts per degree.

/*modified with wide tires increases track width by 0.5 inch
 which is 6.9% more.   Mpy counts by 1.069 to get new counts. 
 360 deg = 976 counts.  2.71  counts per degree.
 
 */



void rotate(int degrees, int speed, char dir, char option)
{   
	
	long dcnt;
	
	int spd,  stable;
	long ss,last;
	
	
	dcnt = (degrees * Cdeg) + ((degrees * Cfrac) / 100);   //
	dcnt++ ;  //Add one to compensate for mechanical backlash etc
	
	
	stable = 0;	
	last = 0;
	spd = 10;
	timer_ramp = 0;
	
	if(timer_ramp == 0){				//Ramp up speed slowly when starting
		if(spd < speed) spd++;
		timer_ramp = 30;		//6ms per speed step
	}
	
	
	if(dir == CW){				//Set direction to rotate  CW or CCW
		cli();
		l_speed = spd;
		r_speed = -spd;
		trip_odo_left = -dcnt;
	    trip_odo_right = dcnt;
		} else {
			l_speed = -spd;
			r_speed = spd;
			trip_odo_left =  dcnt;
			trip_odo_right = -dcnt;
			
			}
		
	
		timer10ms = degrees * 3; 		  //Allow 30ms time per degree for turn
		if(degrees < 30) timer10ms = 200;  //900ms minimum
		sei();
	
		while(getAtomicInt(&timer10ms) > 0){    //Allow max time for this to happen so it won't get stuck ... ever.
			
			timer_ramp = 0;
			if(timer_ramp == 0){				//Ramp up speed slowly when starting
				if(spd < speed) spd++;
				timer_ramp = 50;
			}
			
		    
			refresh_sensors();	
				
			if(option == 1){
			    
				refreshIRdata();
				if((activeIR.brightness > 0x30) && (abs(activeIR.centroid) < 10)){
					timer10ms = 0;  //Quit now if object in front of bot.
					}
			}
			
							
			
			//Now let the trip odometer error term totally control the motors like a servo.
			//Eventually the trip odomenter errors will become zero and the motors will stop.
			
			cli();
			l_speed = -hard_limit_range_l(trip_odo_left*2,spd,-spd);			
			r_speed = -hard_limit_range_l(trip_odo_right*2,spd,-spd);
			sei();
			
			
						
			if(getAtomicInt(&timer100us) == 0){
				cli();
				timer100us = 100;    //10ms				
				ss = abs(trip_odo_right) + abs(trip_odo_left);					
				if(ss != last) stable = 0; else stable++;	//It's been stable for this many 10ms intervals				
				if(stable > 3) timer10ms = 0;  //Exit if trip odometer stable for 30ms
				//if((trip_odo_right == 0) && (trip_odo_left == 0)) timer10ms = 0;
				sei();
				last = ss;
				
			}
			
			
			set_motor_speed(l_speed, r_speed);	
			
			if(option == 2){
					if((abs(r_speed) < 20) && (abs(l_speed) < 20)) timer10ms = 0;
					}
							
			
		}
		
		//Finished turning
		l_speed = r_speed = 0;				//Set motor speeds to zero
		set_motor_speed(l_speed, r_speed);
		if((option & 4) == 0) delay(TMSTOP);						// Wait for motor to stop
	    
		
		
}

//---------------------------------------------------------------------------


void rotateGyro(int degrees, int speed, char dir, char option)
{   

	 long W = 2802 ;  //Integrator counts per degree of rotation

	long angle = degrees * W;
	long gyroI = 0;
	long angle_remaining;
	
	int spd;

if(dir == CCW) set_motor_speed(-speed,speed);
else set_motor_speed(speed,-speed);
	
	angle_remaining = (gyroI  - angle) / 2048;
	
	while(abs(angle_remaining) > 0){  
	
		if(gyro.timer() == 0){
			gyro.timer(5);						//Reset 1ms timer for 5ms
			gyro.read();						//Get latest data from gryo
			if(dir == CCW) gyroI += gyro.z();	//Add gyro z axis data to integrator
				else gyroI -= gyro.z();
			
		}
		
		angle_remaining = (gyroI  - angle) / 2048;
		
		spd = -hard_limit_range_l(angle_remaining,speed,-speed);	//Reduce speed as we approach angle
		if(abs(spd < 15)){
			if(spd >= 0) spd = 15; else spd = -15;		//set min speed so we don't stall before reaching angle
		}
			
		if(dir == CCW) set_motor_speed(-spd,spd);
			else set_motor_speed(spd,-spd);
				
	}
	
	
	set_motor_speed(0,0);
	/*
	debug[0] = gyroI >> 24;
	debug[1] = gyroI >> 16;
	debug[2] = gyroI >> 8;
	debug[3] = gyroI & 255;
	debug[5] = speed;
	
	refreshDebug();
	 */
	}

//----------------------------------------------------------------------------

void Turn(int degrees, int speed )
{
	long   dcnt;   //destination count  36.9 counts/inch 
	long   d;
	
	dcnt = (long)abs(degrees);
	dcnt = (dcnt * 1970) / 720;
	
	
	cli();
	trip_odo_left = 0;
	trip_odo_right = 0;
	sei();
	
	if(degrees > 0){		
		set_motor_speed(speed,-speed);  
		do{
			cli();
			d = dcnt - trip_odo_left;
			sei();
		}while(d > 0);
		
		
	}else{		
		set_motor_speed(-speed,speed);
		do{ 
			cli();			
			d = dcnt - trip_odo_right;			
			sei();
		}while(d > 0);
	}
	
		
	set_motor_speed(0,0);
	
	
	
}

//-----------------------------------------------------------------------------

/* DEAD RECKONING  Switch position = 2
 Rear skid is used as a reference and must be on the start dot.
 Rear skid is 5 inches behind wheels so this must be taken into account when computing distances.
 
 1. Drive 43 inches to first point.  (48-5)
 2. Turn 120 degrees clockwise
 3. Drive 48 inches to second point.
 4. Turn 120 degrees clockwise.
 5. Drive 53 inches to original start point.   (48+5)
 
 */

void deadReckoning()
{   int low_spd;
	
	delay_start(500,deadrec);  // 5 sec delay
	refresh_sensors();
	gyro.begin();
	gyro.zero();
						
	int s = pot / 2;
	low_spd = s / 2;
	int adj = pot2 / 8;
	
	//rotate(360,s,CW,0);
	/*
	driveStraight(120,s,'F');
	odometer_debug();
	set_motor_speed(0,0);
	while(1 == 1) low_spd++;
	*/
	/*
	driveStraight(43,s,'F');
	//rotate(121,low_spd,CW,0);
	//driveStraightGyro(48,s,'F');
	set_motor_speed(0,0);
	wait1ms(100);
	rotateGyro(121,low_spd,CW,0);
	driveStraight(48,s,'F');
	set_motor_speed(0,0);
	wait1ms(100);
	rotateGyro(121,low_spd,CW,0);
	driveStraight(53,s,'F');
	set_motor_speed(0,0);
	*/
	
	
	driveStraightGyro(43, s,'F');
	wait1ms(100);
	rotateGyro(105+adj,low_spd,CW,0);
	
	wait1ms(100);
	driveStraightGyro(48,s,'F');
	wait1ms(100);
	rotateGyro(120,low_spd,CW,0);
	
	wait1ms(100);
	driveStraightGyro(53,s,'F');
	
	debug[0] = pot2;
	set_motor_speed(0,0);
	refreshDebug();
	while(1 == 1);
	
}



//-----------------------------------------------------------------------------

/* BEACON KILLER   Switch position = 1
 
 X 1. Spin around 360 degrees looking for beacon and remember which direction.
 2. Rotate back to point at beacon and start running forward.  (uses quadrature encoders)
 3. Steer towards beacon light unless obstacle is seen.
 4. Steer away from obstacle until it is no longer seen.
 X 5. When beacon intensity reaches a certain level turn off the obstacle avoid logic.  ( < 2 feet from beacon)
 6. When obstacle avoidance is off and the light intensity drops below a threshold assume beacon 
	off switch was hit and stop.
 
 */

//Note the nobeacon level rises with motor noise.  0x20 is with motors running and beacon off.  
//Measured noise with motors on is around 0x1e
#define NOBEACON 0x20

int beaconFinder(char mem)
{   uint8_t no_beacon_ctr;
	int speed, c, x, changeBright;
	long distance, beyond_near, past_obstacle;
	uint8_t near,slow,stop, obstacle, lastBright;
	
	delay_start(500,beaconfinder);
	
	no_beacon_ctr = 0;				//Counts up every 20 ms when beacon signal is not detected
	
	x = face_beacon(0,mem);	
	if(x < NOBEACON) return -1;   //Error.  face_beacon() did not find a beacon.
	
	refresh_sensors();
	refreshIRdata();
		
	cli();
	trip_odo_right = trip_odo_left = 0;   //Zero the odometers when we point directly towards beacon to get direction reference.
	sei();
	distance = beyond_near = 0;
	past_obstacle = 0;
	obstacle = 0;
	
	near = FALSE;
	stop = FALSE;
	slow = FALSE;
	timer10ms = 10;
	lastBright = beacon.brightness;
	
	while(stop == FALSE){
		refresh_sensors();
		refreshIRdata();
		if(beacon.brightness > 0x70) slow = TRUE;		
		if(slow == FALSE) speed = pot / 2; else speed = pot / 3;		
		
		cli();
		if(near == FALSE)distance = trip_odo_left + trip_odo_right;  //"distance" variable will have distance traveled up "near" point.
		beyond_near = trip_odo_left + trip_odo_right - distance;	//"beyond" variable has distance traveled since we got "near" the beacon
		sei();
		
		c = beacon.centroid * 4;   //Get beacon heading
		
		
		//Drive towards the beacon (default mode)
		l_speed = speed + c;
		r_speed = speed - c;
		
		
		
		//Compute active IR obstacle steering
		c = activeIR.centroid ;
		
		if(c < 0){			//Split centroid into left and right "avoid" functions 	 
			c += 0x50; 
			if(c < 0) c = 0;
		}
		else{
			c -= 0x50;
			if(c > 0) c = 0;
		}
		
		c = c * 2;
		
		//Subsume motor control from beacon steering if obstacle present...
		//... and drive around obstacles.
		
		if(activeIR.alarm == 1) {			
			
				l_speed = speed - c;		//Use "avoid" function
				r_speed = speed + c;
				cli();
				past_obstacle = trip_odo_left + trip_odo_right + (CPI*12) ;   //where we are + 6 inches	
				sei();
		} 
		
		
		if(activeIR.alarm == 0){
			if((getAtomic(&trip_odo_left) + getAtomic(&trip_odo_right)) < past_obstacle){
				l_speed = speed;
				r_speed = speed;
			}
		}
			
				
		set_motor_speed(l_speed,r_speed);   //Set motor speeds to computed values, either towards beacon or away from obsticals.
						
		if(beacon.brightness > 0x60)  near = TRUE;		
				
		if(timer10ms == 0){   /*Detect sudden drop in beacon brightenss and stop */
			timer10ms = 2;   //20ms between samples
			changeBright = lastBright - beacon.brightness;
			if(changeBright > 0x20)  stop = TRUE;
			if(beacon.brightness < NOBEACON) {no_beacon_ctr++; } else {no_beacon_ctr = 0;}  //Increment counter when no beacon signal
			if(no_beacon_ctr > 10)	stop = TRUE;   //Stop if no beacon for 200ms
			lastBright = beacon.brightness;
		}
		
	}
	

	return 0;
	
}
	
	
	
//-----------------------------------------------------------------------------

/* LINE FOLLOWER, basic and advanced,  Switch = 0 
 
 1.  Use PID servo loop to follow 3/4 inch black line on white background.
     
     The PID parameters are stored in the "pid_line" struct and initialized 
	in main when the program starts.
 
	Speed is set with the pot.
 
 */

void lineFollow()
{   int u, turnSpeed;
	int gain, last_centroid;
	uint8_t  stop;
	
	delay_start(500,linefoll);  //5 sec delay
	run_seconds = 0;
	last_centroid = 0;
	stop = FALSE;
	timer10ms = 70;			// initialize lost line timer.
	
	
	while((run_seconds < 90) && (stop == FALSE)){   
		
		if(getAtomicInt(&timer10ms) == 0) stop = TRUE;  //Stop if line lost for a long time.  
		
		refreshLineData();		
		refresh_sensors();
		speed = pot / 2; 
		gain = 0x49; //pot2;				//To tune gain - 0x49 works well
		turnSpeed = (speed * 2) / 3 ;
		
		/* If a line is present track it */
		if(line.present == TRUE){  //If we see a line, track it		
			
			if(line.contrast > 0x40){
				
				u =  computePID(line.centroid,&pid_line);  //Update the PID feedback loop.
				u = (u * gain) / 256;	//gain 0x49  speed 0x64 work well
				
				if(speed > 105) speed = speed - abs(u);	//Slow down in turns
				
				r_speed = speed + u;					//Set motor speed values.				
				l_speed = speed - u;
				last_centroid = line.centroid;
			    timer10ms = 70;							//Reset 700ms lost line timer
			}	
			
			
				
		} 
	
		/* If line lost  use last known centroid to track */
		
		if(line.present == FALSE ){
			u =  computePID(last_centroid,&pid_line);  //Update the PID feedback loop based on last valid centroid.
			u = (u * gain) / 256;						
			if(u > 0) u = turnSpeed ; 
				else u = -turnSpeed;					//Fixed turn speed either left or right
			r_speed = u ;								//Set motor speed values.
			l_speed = -u; 		
		}
		
		
		set_motor_speed(l_speed, r_speed);  //Now apply the speed values to the motors .
		
	}
	
}



//---------------------------------------------------------------------------

/* BULLDOZER   Switch = 3
 
 1. Start with front edge sensors hanging off the table facing away from objects.
 2. Measure edge sensor values for floor.
 3. Backup 6 inches and measure the edge sensor values for the table.
 4. Compute a threshold between floor and table.
 5. Rotate CW 113 degrees and go until an edge or object is detected.
    If edge detected rotate CW 160 degrees.  Continue to zig zag until object is seen.
 6. When object is seen using active IR, steer towards it and run until an edge is detected.
 7. Do a backup, rotate, forward, backup until both edge sensors are off the table.  
    Object should be pushed off.
 8. Back up, rotate, continue zig zag pattern while looking for objects.
 
 Note that bot can get lost after steering towards objects and 
 start zig zagging the long table dimension but it still works.

*/
void bulldozer()
{   int sliceL, sliceR, floorL, floorR, tableL, tableR, c ;
    uint8_t  d , deg, object_captured, object_in_view;
	uint8_t edge, expected_edge, edge_status;
	
	
	delay_start(500,bulldoz);  //5 sec delay
	
	refresh_sensors();		
	speed = pot / 2;
	
	run_seconds = 0;
	edge = NONE;			//Neither edge detector triggered yet.
	expected_edge = LEFT;	//We expect the left edge first.
	edge_status = NONE;
	object_captured = FALSE;
	
	refreshEdgeSensors();
	floorL = edge_left  ;  //get light level looking at the floor
	floorR = edge_right   ;
	
	
	trip_odo_left = CPI * 6;   //Setup to back up 6 inches
	while(trip_odo_left > 0) set_motor_speed(-60,-60);		// FIX THIS
	set_motor_speed(0,0);
	delay(20);
	refreshEdgeSensors();
	tableL =  edge_left  ;  //get average table light level
	tableR =  edge_right  ;
	sliceL = floorL + ((tableL - floorL) / 5) ;   //compute slice value 20% up from dark
	sliceR = floorR + (( tableR - floorR) / 5) ;
	
	d = CW;
	deg = 113;
	rotate(deg,speed,d,0);		//Rotate 113 deg
		
	
	
	while(run_seconds < 90){  //Limited to 90 seconds
		refresh_sensors();		
		speed = pot / 2;
		
		
		refreshIRdata();
		
		c = activeIR.centroid ;   //Get object heading
		
		if(activeIR.brightness >= 0x30)  // object whithin 8 inches can be seen 
			object_in_view = TRUE;
			else object_in_view = FALSE;
		
		if(activeIR.brightness >= 0x90)  //Really bright object means we probably captured it
				object_captured = TRUE;
				else object_captured = FALSE;
		
				
		
		if(activeIR.brightness < 0x20) c = 0;  //Ignore far off objects, they may be hallucinations or circuit noise.
		l_speed = speed - c;	//Drive towards the object
		r_speed = speed + c;
		
		refreshEdgeSensors();
		if((edge_right < sliceR) && (edge_left < sliceL)) edge_status = BOTH;
		if((edge_right < sliceR) && (edge_left >= sliceL)) edge_status = RIGHT;
		if((edge_right >= sliceR) && (edge_left < sliceL)) edge_status = LEFT;
		if((edge_right >= sliceR) && (edge_left >= sliceL)) edge_status = NONE;
		
		
		if(edge_status == BOTH){   //Edge detected with both left and right sensors
		    
			set_motor_speed(-speed,-speed);
			trip_odo_left = CPI * 4;
			while(trip_odo_left > 0) ; //Reverse 4 inches   FIX THIS
			
			
			set_motor_speed(0,0);		
			
			switch( edge){
			case NONE:	deg = 180, d = CCW ; break;
			case LEFT:	deg = 160, d = CW; break;
			case RIGHT:	deg = 160, d = CCW; break;
			}
			
			
			rotate(deg,speed,d,1);  //Now make the turn
				
			
			} 
						
			if((edge_status == RIGHT) && (object_captured == FALSE)){
				set_motor_speed(-speed,-speed);
				trip_odo_left = CPI * 4;
				while(trip_odo_left > 0) ; //Reverse 4 inches FIX THIS
				
				if(object_in_view) rotate(30,speed,CCW,0); //If we can see a distant object do not rotate so far as to loose sight of it.
					else rotate(100,speed,CCW,0);  // If not, go ahead and rotate 100 degrees to zig zag.
				
				edge = edge_status = NONE;
				expected_edge = LEFT;
				}
				
			if((edge_status == LEFT) && (object_captured == FALSE)){
				set_motor_speed(-speed,-speed);
				trip_odo_left = CPI * 4;
				while(trip_odo_left > 0) ; //Reverse 4 inches  FIX THIS
				
				if(object_in_view) rotate(30,speed,CW,0);  //If we can see a distant object do not rotate so far as to loose sight of it.
					else rotate(100,speed,CW,0);
				
				edge = edge_status = NONE;
				expected_edge = RIGHT;
				}
				
				
			
		   //Edge detected with either edge sensor (Exclusive OR)
		   //This causes the bot to square itself with the edge to ensure the object is pushed off
		   
			if((edge_status == RIGHT) && (object_captured == TRUE)){  
				r_speed = -40; 
				l_speed = 40;  
				
				edge = edge_status;  //Detect which side went off first
				
			}
			
			if((edge_status == LEFT ) && (object_captured == TRUE)) { 
				l_speed = -40;	
				r_speed = 40;  
				 
				edge = edge_status;  //Detect which side went off first
				
			}
			
			
		
				
		set_motor_speed(l_speed,r_speed);   //Set motor speeds to computed values		
			
		
		}
	
	
	 
	
	
}
//-----------------------------------------------------------------------------
void debugStop()
{
	set_motor_speed(0,0);
	refreshDebug();
	while(1==1){};
}

//----------------------------------------------------------------------------
//Remove all the spaces from a string
void removeSpaces(char *cp)
{
	char *p1 = cp;
	char *p2 = cp;
	p1 = cp;
	while(*p1 != 0) {
		if(*p1  == ' ') {
			++p1;
		}
		else
			*p2++ = *p1++; 
	}
	*p2 = 0; 	
}

//----------------------------------------------------------------------------
/* codes: U: U-Turn  R: Right  L: Left  S: Straight   N: Noose loop  
   if successful there should be no N or U in the final solution.  */

void solveMaze(char *cp, char rule)
{	
	char *p = cp;	
	
	/* Replace the 3 char sting on the left with the single char on the right.  Repeat until none of the
	   3 char strings exist anymore.
	 */
	
	while(*p != 0){
		
		if(strncmp(p,"RUL",3) == 0) { memcpy(p,"U  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"LUR",3) == 0) { memcpy(p,"U  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"SUS",3) == 0) { memcpy(p,"U  ",3);  removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"LUL",3) == 0) { memcpy(p,"S  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"RUR",3) == 0) { memcpy(p,"S  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"SUR",3) == 0) { memcpy(p,"L  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"RUS",3) == 0) { memcpy(p,"L  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"SUR",3) == 0) { memcpy(p,"L  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"LUR",3) == 0) { memcpy(p,"R  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"LUS",3) == 0) { memcpy(p,"R  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"SUL",3) == 0) { memcpy(p,"R  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"RNR",3) == 0) { memcpy(p,"L  ",3); removeSpaces(p); p = cp;  continue;}  
		if(strncmp(p,"LNL",3) == 0) { memcpy(p,"R  ",3); removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"SNL",3) == 0) { memcpy(p,"U  ",3);  removeSpaces(p); p = cp;  continue;}  
		if(strncmp(p,"LNS",3) == 0) { memcpy(p,"U  ",3);  removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"RNS",3) == 0) { memcpy(p,"U  ",3);  removeSpaces(p); p = cp;  continue;}
		if(strncmp(p,"SNR",3) == 0) { memcpy(p,"U  ",3);  removeSpaces(p); p = cp;  continue;}
		p++;
	}
		
	int l = strlen(cp) + 1;
	if(rule == RIGHT) eeprom_write_block (cp, (void *)MAZE_R, l) ; //Save in EEPROM
	if(rule == LEFT)  eeprom_write_block (cp, (void *)MAZE_L, l) ; //Save in EEPROM
	printlcd(cp);
	
	return ;
}
//----------------------------------------------------------------------------


#define L (int) -90
#define R (int)90
#define S (int)0
#define U (int)180
#define NOOSE (int)9997
#define X (int)9998
#define END (int)9999


int computeTurn(char rule, char left_flag, char right_flag, char line, char* nct)
{
	int turn = S;
	
		
	if(line == FALSE){  /* Dead end or T or L intersection found*/
		
		if((left_flag == TRUE) && (right_flag == FALSE)) { turn = L; *nct = TRUE;}//Left turn corner	
		if((left_flag == FALSE) && (right_flag == TRUE)) { turn = R; *nct = TRUE;}//Right turn corner
		
		if((left_flag == TRUE) && (right_flag == TRUE)){ // T intersection choice...
			if(rule == RIGHT) { turn = R; }  //right
			if(rule == LEFT)  { turn = L; } //left					
			
			
		}
		
		if((right_flag == FALSE) 
		   && (left_flag == FALSE) 
		   && (line == FALSE) 
		   && (rule != MEM)){  //  Dead end found, turn around
			turn = U;							
		}						
	}
	
	
	if(line == TRUE){  /*   4 way  or Left T or Right T intersection found */
		
		if((rule == RIGHT) && (right_flag == TRUE)) { turn = R; }					 
		if((rule == LEFT) && (left_flag == TRUE))   { turn = L; }
	}
	
	
	
	if((rule == MEM) && (*nct == FALSE)){ /* Memory mode overrides everything above except no-choice turns*/
		debug[0] = choice;
		debug[1] = saveMaze[choice];
		int path = saveMaze[choice++];
		if(path == 'R') turn = R;
		if(path == 'L') turn = L;
		if(path == 'S') turn = S;
		if(path == 'U') turn = U;
	}
	
	
	
	return turn;
	
	
	
}

//---------------------------------------------------------------------------

//Defines for "dot" direction of travel variable
#define N 1
#define E 2
#define So 3
#define W 4
//----------------------------------------------------------------------------

/* returns directon of travel (dot) based on current dot and turn (S,L,R,U) */
char changeDirection(char dot, int turn)
{
	
	if(turn == S) return dot;
	
	if(turn == L){
		switch (dot){
		case N: dot = W; break;
		case W: dot = So; break;
		case So: dot = E; break;
		case E: dot = N; break;
		}		
	}
	
	if(turn == R){
		switch (dot){
		case N: dot = E; break;
		case E: dot = So; break;
		case So: dot = W; break;
		case W: dot = N; break;
		}
	}
	
	if(turn == U){
		switch (dot){
		case N: dot = So; break;
		case So: dot = N; break;
		case E: dot = W; break;
		case W: dot = E; break;
		}
	}
	
	return dot;
	
}


//----------------------------------------------------------------------------

/* Switch Position 4 & 5 & C :  Line maze.

 Pos 4 is  Right hand follow rule.  
 1. If dead end, turn around 180 degrees
 2. If choice is left or straight, go straight
 3. If choice is right or straight, go right.
 4. If choice is left or right, go right.
 
 5. A fat line indicates we are done.
 
 Pos 5 is left hand rule.  
 Pos C is run maze from eeprom memory
 
*/

//Define fat_line_counter threshold in 1/4 inch units ( 7 = 1 3/4 inch)
#define FAT 7




char* lineMaze(char rule)
{
    int maxfat = 0;	
	int  i,ic, tc, speedo ;
	unsigned int n;
	long dest,fatcheck;
	
	char left_flag, right_flag, dot;
	
	int turn = X;		//Turn in degrees S=0, R=90, L=-90,U=180 X=undefined, END=9999=end of maze
	char path = 0;		//S,U,R,L ascii characters to store in maze memory
	char nct = FALSE;	//Flag for "no choice" turn
	char fat_line_counter = 0;
	char noose = FALSE;
	
	
	
	for(i=0;i<6;i++) debug[i] = 0;			//Clear debug memory
	for(i=0;i<SMSIZE;i++) saveMaze[i] = 0;   //Clear the maze path memory with all zeros
		
	delay_start(500,linemaze);  //5 sec delay
	
	
	if(rule == MEM){
		eeprom_read_block((void*)saveMaze, (const void*)MAZE_R, SMSIZE); //read right hand data into ram
		n = strlen(saveMaze);
		eeprom_read_block((void*)saveMaze, (const void*)MAZE_L, SMSIZE); //read Left hand data into ram
		if(n < strlen(saveMaze))		//Keep left or reload right depending on which is shorter
			eeprom_read_block((void*)saveMaze, (const void*)MAZE_R, SMSIZE);
		
		printlcd(saveMaze);		//Print memory path to LCD
	}
	
	ic = 0;	
	run_seconds = 0;
	n = 0;
	
	tc = 0;
	i = 0;
	choice = 0;
	
	dot = N;
	xDist = 0;
	yDist = 0;
	
	
	cli();
	trip_odo_left = 0;
	trip_odo_right = 0;
	odo_right = 0;
	odo_left = 0;
	fatcheck =   CPI/2;           //1/4 inch ahead
	sei();
	
		
	left_flag = right_flag = FALSE;
	timer_trip = 3 ; //30ms
	
	refresh_sensors();		//Read speed pot.
	speed = pot / 2;		//Set speed
	int turnSpeed = 60 ; //This is about as fast as we can turn and not fail
	
	//Maze loop
	while((run_seconds < 90 ) && (turn != END)){    //Run until END or 90 seconds max.
		
		sbi(PORTC,7);						//Light debug LED on every loop			
		refreshLineData();		
		
		cbi(PORTC,7);						//LED off

		trackLine(speed);
		
		
		/* End Square Detection */
		cli();
		if(fatcheck <= (odo_right + odo_left)){				//Check for fat line every 1/4 inch
			if(line.width >= 5) fat_line_counter++;	//advance fat line counter if fat line seen
				else fat_line_counter = 0;
			if(fat_line_counter >= FAT) turn = END;	
			if(fat_line_counter > maxfat) maxfat = fat_line_counter;	//remember highest value for debugging	
			fatcheck = odo_right + odo_left + CPI/2;		// 1/4 inch ahead we will look  again
			ic++;
			switch (dot){
			case N: yDist++; break;
			case So: yDist--; break;
			case E: xDist++; break;
			case W: xDist--; break;
			}
		}
		sei();
		
		if(turn == END ) {				//Found 4x4 inch end square				
			set_motor_speed(0,0);		//Stop, all done.					
			break;
		}		
		
		
		
		/* Intersection detecton and Left or right turn process */	
		
		turn = X;  //undefined turn
		
		
		/* Note the line_lost_timer is set at 100ms to pass over gaps up to 1.5 inches in the line (11/8/2012)  */
		if((line.map & 0x81) || (line.lost_line_timer > 100)  ){ //We see a turn or intersection! 			
			
			set_motor_speed(speed,speed);						// proceed straight.
			left_flag = right_flag = FALSE;						//reset intersection flags	
					
			cli();										//Interrupts must be off when we deal with trip_odo_*
			trip_odo_right = 0;	
			speedo = (l_speedometer + r_speedometer) / 2;	//Actual speed derived from wheel encoders			
			int z = 200 - ((speedo*3) / 2);					//Compensate for stopping distance vs speed
			if(z < CPI) z = CPI;
			dest = trip_odo_right + z;
			
			sei();										//Interrupts on again
			turn = X; // X is undefined
			path = 0;
			nct = FALSE;  //Default to no-choice turn type
			i = 0;
			
			while((i == 0) || (line.width >= 5)) {
								
				if(line.map & 0x80)  left_flag = TRUE;		//Set left_flag if we see a line on the left
				if(line.map & 1)    right_flag = TRUE;		//Set right_flag if we see a line on the right
				
				cli();									//Disable interrupts
				if(dest <= trip_odo_right) i = 1  ;		//Set i to 1 when destination reached	
				if(fatcheck <= (odo_right + odo_left)){				//Check for fat line every 1/4 inch					
					if(line.width >= 5)  fat_line_counter++;	//advance fat line counter if fat line seen	
					if(fat_line_counter > maxfat) maxfat = fat_line_counter; //remember highest value for debugging
					if(fat_line_counter >= FAT) { turn = END; i = 1;}  //All done when end square found
					fatcheck = odo_right + odo_left + CPI/2;		//1/4 inch ahead we will look  again
					ic++;											//update total 1/4 inches traveled for debugging
					switch(dot){
					case N: yDist++; break;
					case So: yDist--; break;
					case E: xDist++; break;
					case W: xDist--; break;
					}
				}
				sei();									//Enable interrupts again
						
					
				trackLine(speed);   //track line if present
										
			}  //end while(i==0) loop
			
			tc++;		// Count all turns for debug
			
			if(turn != END)	{
				turn = computeTurn(rule,left_flag,right_flag,line.present,&nct);  //Compute turn now.
												
				if(turn != S){					
					Turn(turn,turnSpeed);				//Now do the physical turn.	
					searchForLine(turn,20,turnSpeed);	//Find the line
					centerOnLine(10,turnSpeed);			//Center on the line
					
					
				}
				
				dot = changeDirection(dot,turn);				
				
				if((abs(yDist) < 12) && (abs(xDist) < 12)) 	noose = TRUE; 
				   else noose = FALSE;					
				if(nct == FALSE) { 
					yDist = 0;
					xDist = 0;
				}
			}
			
	
			
				
			refreshLineData();	
			
			if((line.present != TRUE) &&  (turn != END)){	//Debug trap for failed turns , ie: no line detected after turn.
				set_motor_speed(0,0);
				debug[0] = ic & 255;
				debug[1] = maxfat;
				debug[2] = turn & 255;
				debug[3] = left_flag;
				debug[4] = right_flag;			
				debug[5] = speed; 
				refreshDebug();			
				while(1==1){};
			}
			/*
			if(tc >= 7){
				debug[0] = xDist ;
				debug[1] = yDist;
				debug[2] = dot;
				debug[3] = noose;
				refreshDebug();
				set_motor_speed(0,0);
				while(1 ==1){};
			} */
			
			/* Convert the turn code into ASCII characters and save in turn memory */
			if((turn != END) && (nct == FALSE) && (rule != MEM)) {
				switch(turn){								
					case S: path = 'S'; break;					
					case R: path = 'R'; break;
					case L: path = 'L'; break;
					case U: path = 'U'; break;
					//case NOOSE: path = 'N'; break;
					default: path = 0; break;
				}
				
				if(choice < SMSIZE-1){
					if(noose == TRUE){ 
						saveMaze[choice] = 'N';   //Save noose flag if true
						if(choice < SMSIZE-1) choice++;
					}
					saveMaze[choice] = path;					//Save turn in memory
					if(choice < SMSIZE-1) choice++;				//Increment turn memory pointer
				}
					
				path = 0;
											
			} 
			
				
			left_flag = FALSE;
			right_flag = FALSE;			
			l_speed = r_speed = speed;
			
		} //End of intersection code
			
		
	}  //DONE!  End of loop.
		
	set_motor_speed(0,0);		//Stop motors
	solveMaze(saveMaze,rule);   //Now solve the maze and save to EEPROM .
	return saveMaze;
	
}  //End of maze code.





		

//---------------------------------------------------------------------------

/* Switch Position F :  Debug Code to display Active IR object sensor values */

void test_sensors(void)
{	uint8_t adr = 0;

	unsigned int ctr,x;
	
		
	
	sbi(PORTC,MUXENABLE);		//Disable IR LEDs
	x = 0 ;
	
	while (1 == 1){
		
		irsensor[adr] = readActiveIR(adr);
		sbi(PORTC,MUXENABLE);
		_delay_ms(1);			//About 1 ms between pulses
		adr++;					//Next LED address
		x++;
		
		if (x >= 100){  //about 10 updates per sec
			for(ctr=0;ctr < 6;ctr++) debug[ctr] = irsensor[ctr];
			
			refreshDebug();
			x = 0;
			}
		}
		
		



}

//--------------------------------------------------------------------------

/* Active IR test readings.

Left and right edge sensors.
1. White paper		0x98
2. Brown table top  0x40
3. Off edge, beige carpet 0x00
4. Off edge, white paper  0x0a
5. Black tape, 3/4 inch   0x20


*/

//Switch = A
void test_ir()
{   int c;
	
	while(1 == 1){
		refreshIRdata();
		refreshEdgeSensors();
	
		debug[0] = activeIR.centroid;
		debug[1] = activeIR.map ;
		debug[2] = activeIR.brightness;
				
		c = activeIR.centroid ;
		if(c < 0) c += 0x50; else c -= 0x50;
		debug[3] = c & 255;
		
		debug[4] = edge_left;
		debug[5] = edge_right;

		
		delay(15);
		refreshDebug();
		
	}
	
	
}




//---------------------------------------------------------------------

//Switch position = B

void test_beacon()
{
	
	
	//set_motor_speed( 64,64);  // uncomment to run motors for noise testing
	
	while(1 == 1){
		refresh_sensors();	
		
		debug[0] = beacon.left;
		debug[1] = beacon.right;
		debug[2] = beacon.brightness;
		debug[3] = beacon.centroid & 255;
		debug[4] = 0;
		debug[5] = beacon.left + beacon.right - abs(beacon.right-beacon.left);
		
		delay(10);
		if(debugReady()) refreshDebug();
		
		sbi(PORTC,7);
		testfloat = (double)beacon.right / ((double)beacon.left + 300.0) ;
		cbi(PORTC,7);
		refresh_sensors();
		sbi(PORTC,7);
		testfloat = (double)beacon.right * ((double)beacon.left + 300.0) ;
		cbi(PORTC,7);
		refresh_sensors();
		sbi(PORTC,7);
		testfloat = (double)beacon.right + ((double)beacon.left + 300.0) ;
	    cbi(PORTC,7);
		
	}
	
	
	
}



//Switch position = E
/*
void test_passive_sensors()
{

		
	while(1 == 1){
		refreshLineData();
		debug[0] = get_adc8(AD_BEACON_LEFT);
		debug[1] = get_adc8(AD_BEACON_RIGHT);
		debug[2] = get_adc8(AD_POT);

		debug[3] = line.data[7];		
		debug[4] = line.data[0]	;	
		debug[5] = get_adc8(AD_MOTOR_CURRENT);
		
		timer10ms = 15;
		while(timer10ms > 0);
		refreshDebug();
		}

}

*/
//Switch position = D

void test_line_photocells()  
{  
	while(1 == 1){
		refreshLineData();
		refresh_sensors();
		refreshEdgeSensors();
		debug[0] = line.brightness;  //Varies from 0x80 (black) to 0xd0 (white)
		debug[1] = edge_left;		//Table edge left
		debug[2] = edge_right;		
		debug[3] = line.contrast;
		debug[4] =  line.centroid & 255;
		debug[5] =  line.map;
		
		refreshDebug();
	}
}


//Switch position E
void test_first_6_photocells()
{ uint8_t i;
while(1 == 1){
		refreshLineData();
		refresh_sensors();
		for(i=0;i<6;i++) debug[i] = line.data[i];
		
		refreshDebug();
	}
}


//--------------------------------------------------------

//Switch pos 8

//Save photocell white levels in EEPROM
//This must be done at least once.
//Bot must be on a white surface with no line!


int saveLineWhiteLevels()
{
    int i;
	uint8_t adr, cksum;
	
	delay_start(500,whitelevel);  //5 sec delay
	cksum = 0;
	//Read photocells.  Assume we are on pure white with no black line 
	for(adr=0;adr<8;adr++){
		if(adr & 1) sbi(PORTC,MUXA0); else cbi(PORTC,MUXA0);
		if(adr & 2) sbi(PORTC,MUXA1); else cbi(PORTC,MUXA1);
		if(adr & 4) sbi(PORTC,MUXA2); else cbi(PORTC,MUXA2);
		line.data[adr] = get_adc8(AD_LINE_FOLLOW);
		
		}
		
	line.data[8] = checksum(line.data,8);
	
     
   eeprom_write_block (line.data, (void *)0, 9) ; //Save values in EEPROM
   eeprom_write_block (line.data, (void *)16, 9) ; //Save values in EEPROM backup location
   
   eeprom_read_block((void*)&line.data, (const void*)0, 9);  //Read back for verification
   
   
   if(checksum(line.data,8) != line.data[8]) return -1;
      
   for(i=0;i<6;i++) debug[i] = line.data[i];
   refreshDebug();								//Show first 6 values on LCD
   run_seconds = 0;
   while(run_seconds < 5);
   return 0;
   


} 

//-----------------------------------------------------------



//-----------------------------------------------------------


// Switch position = C

/*void test_quadrature()

{   int d;
	
	odo_right = 0;
	odo_left = 0;
	delay_start(500,(PGM_P)testquad);  //5 sec delay
	timer_trip = 200;
	d = 0;
	
	while(1 == 1){
		refresh_sensors();
		speed = pot / 2;
		if(timer_trip == 0) speed = 0;
		l_speed = speed;
		r_speed = speed;
		d = odo_right - odo_left;
		if(d > 10) d = 10;
		if(d < -10) d = -10;
		
		l_speed = l_speed + d;
		r_speed = r_speed - d;
		set_motor_speed(l_speed, r_speed);
		
		if(debugReady()){
			debug[0] = (odo_right >> 16) & 255;
			debug[1] = (odo_right >> 8) & 255;
			debug[2] = (odo_right & 255);
			debug[3] = (odo_left >> 16) & 255;
			debug[4] = (odo_left >> 8) & 255;
			debug[5] = (odo_left & 255);
			refreshDebug();
		}
	}
		
		
	
} */

//----------------------------------------------------------------------------
void showLineData()
{

	line.maxwhite = 255;
	line.maxblack = 0;
	while(1 == 1){
	refreshLineData();
	if(debugReady()){
			debug[0] = line.data[0] ;
			debug[1] = line.data[1];
			debug[2] = line.data[2];
			debug[3] = line.data[3];
			debug[4] = line.data[4];
			debug[5] = line.data[5];
			refreshDebug();
		}
		
	}	
		
}

//----------------------------------------------------------------------------


void stopTest()
{
   
	refresh_sensors();
	int speed = pot / 2;
	delay(100);
	refreshLineData();
	set_motor_speed(speed,speed);
	
	do{
		refreshLineData();
	}while(line.width < 4);
	cli();
	debug[0] = l_speedometer >> 8;
	debug[1] = l_speedometer & 255;
	debug[3] = r_speedometer >> 8;
	debug[4] = r_speedometer & 255;
	debug[5] = speed;
	sei();
	refreshDebug();
		
	odo_left = odo_right = 0;
	
	set_motor_speed(0,0);	
	delay(100);
	
	//debug[0] = speed & 255;
	//debug[4] = odo_left & 255;
	//debug[5] = odo_right & 255;
	//refreshDebug();
	
	while(1);
	
	
	
	
	
}
//-------------------------

//Switch pos 9
void testuart()
{    
	char txb[40];
	
	while(1 == 1){
		if(rxstate != 0){
			//for(i=0;i<6;i++) debug[i] = rx_data[i];
			//refreshDebug();
			
			strncpy(txb,rx_data,40);
			sendData(txb);
			rxstate = 0;
			if(rx_data[0] == 0x41) test_sensors();
		}
	}
	
	
}

//---------------------------------------
	
int follow_beacon()
{
	
	int speed, c, x;
	long distance, beyond_near, pass_obstacle;
	uint8_t near,stop, obstacle;
	
	delay_start(500,beaconfinder);
	
	x = face_beacon(0,0);	
	if(x < 0x20) return -1;   //Error.  No beacon found
	refresh_sensors();
	refreshIRdata();
	if(beacon.brightness < 0x1c) return -2 ; // Error - not actually pointing at beacon when it should be.
	
	trip_odo_right = trip_odo_left = 0;   //Zero the odometers when we point directly towards beacon to get direction reference.
	distance = beyond_near = 0;
	pass_obstacle = 0;
	obstacle = 0;
		
	near = FALSE;
	stop = FALSE;
	
	while(stop == FALSE){
		refresh_sensors();
		refreshIRdata();
		
		speed = ( 0x60 - beacon.brightness) * 5;
		speed = hard_limit_range(speed,110,-80);
		if(abs(speed) < 10) speed = 0;
		if(beacon.brightness < 0x30) speed = 0;
		
		
		c = beacon.centroid * 4;   //Get beacon heading
		
		//Drive towards the beacon (default mode)
		l_speed = speed + c;
		r_speed = speed - c;
		
		
 
		
		
		//Compute active IR obstacle steering
		c = activeIR.centroid ;
		
		if(c < 0){			//Split centroid into left and right "avoid" functions 	 
			c += 0x50; 
			if(c < 0) c = 0;
		}
		else{
			c -= 0x50;
			if(c > 0) c = 0;
		}
		
		c = c * 2;
		
		//Subsume motor control from beacon steering if obstacle present...
		//... and drive around obstacles.
		
		if(activeIR.alarm == 1) {			
			
			l_speed = speed - c;		//Use "avoid" function
			r_speed = speed + c;
			//pass_obstacle = trip_odo_left + (CPI*5) ;   //where we are + 5 inches	
			
			
		} //else obstacle = 0;
				
		
		set_motor_speed(l_speed,r_speed);   //Set motor speeds to computed values, either towards beacon or away from obsticals.
		
		
		//distance = (trip_odo_left + trip_odo_right) / (2 * CPI);  //Distance traveled in inches.  
		//May use this someday to determine when to disable object ovoidance
		//instead of beacon brightness.
		
		
		
		
		
	}
	
	set_motor_speed(0,0);
	
	return 0;
	
}

	
	
	
//--------------------------------------
void rotatetest()
{
	wait1ms(1000);
	refresh_sensors();
	
	int spd = pot / 2;
	gyro.begin();
	gyro.zero();
	
	rotateGyro(360,spd,CW,0);
	set_motor_speed(0,0);
	debug[0] = pot2;
	debug[1] = pot /2;
	refreshDebug();
	while(1 == 1);
	
}

//--------------------------------------   
   

void drivetest()
{   
	gyro.begin();
	gyro.zero();
	driveStraightGyro(48,60,'F');	
	wait1ms(200);	
	driveStraightGyro(48,60,'R');
	


}	

//-----------------------------------------
void motortest(char m)
{
	DemoMode = m;     //when m != 0 only one motor will have closed loop speed control.
	while( 1 ==1 ){
		refresh_sensors();
		r_speed = pot - 128;
		l_speed = pot - 128;
		set_motor_speed(l_speed, r_speed);  // set the motor speeds.
		
		if(debugReady()){
			debug[0] = beacon.left;
			debug[1] = beacon.right;
			debug[2] = beacon.brightness;
			debug[3] = (r_speedometer >> 8 ) & 255;
			debug[4] = (r_speedometer & 255);
			debug[5] = pot2;
			refreshDebug();
		}
		
	}
	
	DemoMode = 0;


}
//---------------------------------------------------------------------------
void testMaze()
{
		
	while(1==1){
     refreshEdgeSensors();
		
		debug[0] = edge_left;
		debug[1] = edge_right;
		refreshDebug();

	
	};
	
}



//----------------------------------------------------------------------------

void i2ctest()
{
	
	//long int integrator = 0;	
	
	delay(500);
	
	compass.begin();
	
	
	while(1 == 1){
		compass.read();
		
				
		  debug[0] = compass.x() >> 8;
		  debug[1] = compass.x() & 255;
		  debug[2] = compass.y() >> 8;
		  debug[3] = compass.y() & 255;
		  debug[4] = compass.z() >> 8;
		  debug[5] = compass.z() & 255;
	
		wait1ms(10);
		refreshDebug();
	
	}
	
		
	return;
		
}


	
//---------------------------------------------------------------------------

int main(void)
{ 
  char *sbuf, *s;
  int last_sec, rv;
   
/* Digital ports setup */
   
  PORTD = 0x82;			//PORTD setup
  DDRD = 0x32;
              
  PORTB = 0xff;			// PORTB:  pull ups for selector switch 
  DDRB = 0x10;			// 0..3 inputs for octal selector switch. Bit 4 output for motor enable 
  
  PORTC = 0xfc;			//NO PULLUPS ON BITS 0,1 FOR TWI.  3.3v devices would not like 5V pullups!
  DDRC = 0xfc;			//PORTC all outputs except for 0,1 which are for twi (i2c)
	

/* PORTA: A/D Converter setup */

  DDRA = 0;					// 8 bits analog inputs	
  PORTA = 0;				// Clear port A  
  ADCSRA = BV(ADEN) + BV(ADPS2) + BV(ADPS0); // Enable A to D converter with 16mhz/32 conv clk.  ( 500 KHZ)
  
/* Timer 1 dual PWM setup */

  TCCR0B = BV(CS01);		        // use CLK/8 source for counter 
  TCCR1A = BV(COM1B1) + BV(COM1A1) + BV(WGM10);  // 8 bit pwm  OCnx clear at top
  TCCR1B = BV(CS10);				// clk/1 prescale, 8 bit phase correct pwm  (28.8 khz pwm freq))  
  TCCR1C = BV(FOC1A);				// Force output compare for channel A
  TIMSK1 = 0;						// No interrupts from this timer/coounter
  OCR1AH = 0;						// Clear all output compare registers
  OCR1AL = 128;
  OCR1BH = 0;
  OCR1BL = 128;
  
  

/* Timer-Counter 2 setup */
  TCCR2A = BV(WGM21);			//CTC mode
  TCCR2B = BV(CS21) + BV(CS20);	//Prescale = 32  on TC2 (500 khz with 16mhz osc)  
  OCR2A = 49;					// T/C 2 compare register A for 10,000 hz interrupts (clk/(32*(1+OCR2A))  
  sbi(TIMSK2,OCIE2A);           // Enable timer 2 compare A interrupts

/* UART setup */
  UBRR0L = 103;              /* UART 0 baud rate set to 9600 (16mhz osc) */
  sbi(UCSR0B,TXEN0);              // enable uart 0 transmitter  (interrupts enabled in sendData function)
  sbi(UCSR0B,RXEN0);              // enable uart receiver
  
	
/*External interrupt setup*/
	EICRA = BV(ISC11) + BV(ISC01) ;	//Falling edge interrupts on INT0 and INT1
	sbi(EIMSK,INT1);				//Enable INT1
	sbi(EIMSK,INT0);				//Enable INT0
	
	txstate = 0;
	rxstate = 0;		//Clear receive buffer state to empty
	rx_ptr = 0;			//RX buffer pointer cleared
	
	sbi(UCSR0B,RXCIE0);             // receiver interupt enabled
	/*
	home[0] = 0x1b;
	home[1] = '[';
	home[2] = 1;
	home[3] = ';';
	home[4] = '1';
	home[5] = 'H';
	home[6] = 0;
	*/
	
	home[0] = 0x12;
	home[1] = 0;
	home[2] = 0;
	
	
	trip_counter = 0;
	timer1ms = 0;
	timer10ms = 0;
	timer100us = 0;
	minutes = 0;
	run_seconds = 0;
	runtime = 0;
	last_sec = 0xffff;
	last_sec = 0;
  
  odo_left = 0;
  odo_right = 0;
  line_history_odo_right = 0;
  line_history_odo_left = 0;
	
	l_speedo_counter = 1;
	r_speedo_counter = 1;
	
	activeIR.cntr = 0;
	activeIR.alarm = 0;
	
	line.maxwhite = 255;  //reset the global max white and black so they can be determined in the contrast routine.
	line.maxblack = 0;
  
  l_speed = 0;
  r_speed = 0;
	l_motor = 0;
	r_motor = 0;
	timer_ramp = 0;
  timer10ms = 500;				//Set for 5 second delay
  timer_overcurrent = 100;		//1000ms
  sel_sw = (PINB ^ 0x0f) & 0x0f;		//Read selector switch
  
  /* read white level references from eeprom for line sensors */
	eeprom_read_block((void*)&line.data_wref, (const void*)0, 9);  //Read back stored white level references
	
	if(checksum(line.data,8) != line.data[8]) {
		eeprom_read_block((void*)&line.data_wref, (const void*)0, 9); //Read spare values if main corrupted
		//Need to eventually add code to check the spares too.
	}
  
	
	
	
  sei();                      /* enable global interrupts */
	
	imu.begin() ;                   // initialize i2c and IMU 
  
 // wdt_enable(WDTO_250MS);          /* enable watchdog timer - 250 ms timeout. */ 
	
	cbi(PORTC,7);	//Test LED off
    sbi(PORTC,MUXENABLE);   //IR LEDs off
	cbi(PORTB,MOT_ENAB);   //Disable motors
	set_motor_speed(0,0);
	refresh_sensors();
	
	//line follower PID defaults
	pid_line.kp = 2;		//porportional times 2
	pid_line.ki = 200;		//Integral div by 200
	pid_line.kd = 120;		//Deriviative times 120
	pid_line.t = 10;		//10ms between updates
	pid_line.ilim = 100;	//Limit integral to +/- 100
	pid_line.i = 0;			//Clear all the data
	pid_line.d = 0;
	pid_line.p = 0;
	pid_line.u = 0;
	
	
  delay(10);			//Wait 100ms for LCD to init.	
  clearScreen();
   
	s = (char*)"";
	rv = 0;
  
  switch (sel_sw){			//Run job based on selector switch position
	  case 0:	lineFollow(); break;
	  case 1:	rv = beaconFinder(0); break;
	  case 2:	deadReckoning(); break;
	  case 3:	bulldozer(); break;
	  case 4:	s = lineMaze(RIGHT); break;
	  case 5:	s = lineMaze(LEFT); break;
	 case 6:	motortest(0); break;
	 case 7:    drivetest(); break;
	 case 0x08:	rv = saveLineWhiteLevels(); break;    //White level calibration
	 //case 0x09:	showStack(); break;
	 // case 0x09:  testuart(); break;
	  //case 0x09:	rv = follow_beacon(); break;
	  //case 0x09:   rv = beaconFinder(1); break;    //Beacon finder with memory of initial beacon direction
	  //case 0x09:   testMaze();
	  //case 0x09:  showLineData(); break;
	  //case 0x09: stopTest(); break;
	  //case 0x09:   rotatetest(); break;
	  case 0x09: i2ctest(); break;
	 case 0x0a:	test_ir(); break;
	 case 0x0b:   test_beacon();  break;    //Show beacon sensor data on debug LCD
	  
		  //face_beacon(TRUE,0); break;  //Test the face beacon function
	   // case 0x0c:	test_quadrature(); break;
	  case 0x0c:	s = lineMaze(MEM); break;
	 	  
	  case 0x0d:	test_line_photocells(); break;
	  case 0x0e:   test_first_6_photocells(); break; 	  	  
	  case 0x0f:   motortest(0);
	  //case 0x0f:	test_sensors(); break; 
  
  };
  
 r_speed = l_speed = 0; 
 set_motor_speed(l_speed, r_speed);
 cbi(PORTB,MOT_ENAB);   //Disable motors
 
 clearScreen();
 delay(10);
 sbuf = (char*)"?";
 
	if(rv == 0) {	
		sbuf = (char*)malloc(100);
		sprintf(sbuf,"%1d.%1d\n\r%s",runtime/100, runtime%100,s);		
	}
 if(rv == -1) sbuf = (char*)"Error 1";
 if(rv == -2) sbuf = (char*)"Error 2";
 
 sendData((char*)"\x12\x00");
 sendData(sbuf);
 while(1 == 1);		//Stop here.  Infinite loop.
 return 0;

};

/* End of File */
