/*
 *  imu.cpp
 *  
 *
 *  Created by Dale Heatherington on 12/23/13.
 *  
 *
 */

extern "C" {
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include "twi.h"
}

#include "Wire.h"
#include "imu.h"



//-------IMU-----------------------------------------------------------


//Constructor

Imu::Imu()
{
}

void Imu::begin(void) 
{
	Wire.begin();
}

void Imu::write( int adr, int reg, uint8_t data)
{
	Wire.beginTransmission(adr);
	Wire.write(reg);				
	Wire.write(data);					
	Wire.endTransmission();	
}


void Imu::read(int adr, int reg, int *buf, int nwords, bool lsb_first=false )
{
	Wire.beginTransmission(adr); // i2c device address adr
	Wire.write(reg);			//Start at register "reg"
	Wire.endTransmission();
	Wire.requestFrom(adr,nwords*2);	//Get  bytes starting at reg. 	
	
	uint8_t i=0;
	
	while(( Wire.available() > 0) && (i<nwords)){  /* Reading bytes now... */
		
		if(lsb_first){
			buf[i] =  Wire.read() ;		//Get LSB first
			buf[i] |= Wire.read() << 8;  //Get msb last
		} else{
			buf[i] =  Wire.read();		//Get msb first
			buf[i] =  buf[i] << 8;		//Shift it up to high byte
			buf[i] |= Wire.read();		// OR in the lsb last
		}
		
		i++;
	}
	
	
}




//-------Rate Gyro-------------------------------------------------------------------

int Gyro::adr=0x68	;	//twi address of gyro
int Gyro::values[3] = {0,0,0};    //Holds x,y,z gyro values
int Gyro::offsets[3] = {0,0,0};	//Holds offsets determined when Gyro.zero() is called.
int Gyro::counter = 0;				//Down counter 



Gyro::Gyro()
{
	
};

void Gyro::begin(void)
{
	write(adr,22,0x1a);		//DLPF, full scale
	write(adr,62,3);		//FL sel = 2000 degree/sec full scale,  98Hz low pass bw
}

void Gyro::write(int adr,int reg, uint8_t data)
{
	Imu::write(adr,reg,data);
}

void Gyro::read(void)
{
	Imu::read(adr,29,values,3,false);  //get 3 16 bit words starting at register 29
	for(int i=0;i<3;i++) values[i] = values[i] - offsets[i];  //Subtract offsets
	
	
}

void Gyro::zero(void)  //Call this only when stationary.
{   int i;
	long avg[3];
	for( i=0;i<3;i++) avg[i] = 0L;
	
	for(i=0;i<100;i++){
		Imu::read(adr,29,offsets,3,false);
		avg[0] += offsets[0];		//Add the at-rest gyro readings to avg 100 times
		avg[1] += offsets[1];
		avg[2] += offsets[2];
	}
	
	for(i=0;i<3;i++) offsets[i] = (int)avg[i] / 100;  //Compute the average offsets
}

void Gyro::tick(void)  //Called by periodic interrupt every 1000us  (1000 hz)
{
	if(counter > 0) counter--;   //Decrement counter
	
	
}

int Gyro::timer(void)  //Read time left
{
	return counter;
}

void Gyro::timer(int time)  //Set timer
{
	counter = time;
}

	


int Gyro::x()
{
	return values[0];
}

int Gyro::y()
{
	return values[1];
}


int Gyro::z()
{
	return values[2];
}


//----------Accelerometer----------------------------------------------------------------------

int Accel::adr=0x53 ;				//twi address of accelerometer
int Accel::values[3] = {0,0,0};		//Holds x,y,z accelerometer values
int Accel::offsets[3] = {0,0,0};	//Holds offsets determined when Accel.zero() is called.
int Accel::counter = 0;				//Down counter

Accel::Accel()
{ 
}

void Accel::begin(void)
{
	write(adr,0x31,1);		//Register 0x31:  Range +/- 4G , right justify, 10bit mode
	write(adr,0x2d,8);		//Register 0x2d: Power control, Set to measure mode
	write(adr,0x38,0);		//Register 0x38: Fifo mode, Fifo bypass
}

void Accel::write(int adr,int reg, uint8_t data)
{
	Imu::write(adr,reg,data);
}

void Accel::read(void)
{
	Imu::read(adr,50,values,3,true);  //get 3 16 bit words starting at register 50 (LSB first)
	for(int i=0;i<3;i++) values[i] = values[i] - offsets[i];  //Subtract offsets
	
	
}


void Accel::zero(void)  //Call this only when stationary.
{   int i;
	long avg[3];
	for( i=0;i<3;i++) avg[i] = 0L;
	
	for(i=0;i<100;i++){
		Imu::read(adr,29,offsets,3,false);
		avg[0] += offsets[0];		//Add the at-rest accelerometer readings to avg 100 times
		avg[1] += offsets[1];
		avg[2] += offsets[2];
	}
	
	for(i=0;i<3;i++) offsets[i] = (int)avg[i] / 100;  //Compute the average offsets
}

void Accel::tick(void)  //Called by periodic interrupt every 1000us  (1000 hz)
{
	if(counter > 0) counter--;   //Decrement counter
	
	
}

int Accel::timer(void)  //Read time left
{
	return counter;
}

void Accel::timer(int time)  //Set timer
{
	counter = time;
}


int Accel::x()
{
	return values[0];
}

int Accel::y()
{
	return values[1];
}


int Accel::z()
{
	return values[2];
}








//--------------Compass--------------------------------------------------------------

// HMC5883 3 axis magnetometer compass 
 
int Compass::adr=0x1e;					//address of compass 
int Compass::values[3] = {0,0,0};		//Holds x,y,z accelerometer values
int Compass::offsets[3] = {0,0,0};	    //Holds offsets determined when Compass.zero() is called.
//int Compass::counter = 0;				//Down counter

Compass::Compass()
{	
}
void Compass::begin(void)
{
	write(0,0x70);		//Register 0: Config A, 8 samples averaged, 15 hz update rate, normal mode 
	write(1,0x20);		//Register 1: Config B, Gain = 1090 LSb/Gauss default
	write(2,0);			//Register 2: Mode,  Continous measurment mode
}

void Compass::write(int reg, uint8_t data)
{
	Imu::write(adr,reg,data);
}

void Compass::read(void)
{
	Imu::read(adr,3,values,3,false);  //get 3 16 bit words starting at register 3 (MSB first)
	for(int i=0;i<3;i++) values[i] = values[i] - offsets[i];  //Subtract offsets
	
	
}

void Compass::zero()
{
	//To be implimented.  Will involve rotating the bot 360 to find min and max field values.
}


int Compass::x()
{
	return values[0];
}

int Compass::y()
{
	return values[2];
}


int Compass::z()
{
	return values[1];
}




//---------------------------------------------------------------------------------



/* Create instances of these classes  */
Imu imu = Imu();
Gyro gyro = Gyro();
Accel accel = Accel();
Compass compass = Compass();




